"""
Miscelaneous methods
======================
"""

import numpy



def pol2cart(r, phi):
	"""
Converts from polar to cartesian coordinates.

>>> x,y=pol2cart(r,phi)
	"""
	x = r * numpy.cos(phi)
	y = r * numpy.sin(phi)
	return x, y
    

def cart2pol(x, y):
	"""
Converts from cartesian to polar coordinates.

>>> r,t=cart2pol(x,y)
	"""
	r = numpy.sqrt(x**2 + y**2)
	t = numpy.arctan2(y, x)
	return r, t



def vel_p2c(th,vr,vth):
	"""
Computes the cartesian components of a velocity vector which
is expressed in polar coordinates. i.e. apply a change of 
basis. See for example discussion after eq. 4 in 
https://ocw.mit.edu/courses/aeronautics-and-astronautics/16-07-dynamics-fall-2009/lecture-notes/MIT16_07F09_Lec05.pdf

Returns: vx, vy
	"""
	vx=vr*numpy.cos(th)-vth*numpy.sin(th)
	vy=vr*numpy.sin(th)+vth*numpy.cos(th)

	return vx, vy




def vel_c2p(th,vx,vy):
	"""
Computes the polar components of a velocity vector which
is expressed in cartesian coordinates. 

Returns: vr, vth
	"""
	vr=vx*numpy.cos(th)+vy*numpy.sin(th)
	vth=-vx*numpy.sin(th)+vy*numpy.cos(th)

	return vr, vth

    
	

def evalfun(fun,x,par):
	"""
Evaluates the function fun at each element of the array x, for the parameters
provided in the array/list par. fun is assumed to return a scalar. Returns an
array with fun evaluated at x. See example below.

Usage:

>>> p=array([1,2,3])

  x=1, par0=2, par1=3 
  
>>> fun(p)

  returns a scalar
  
>>> x=linspace(0,10,50)

>>> evalfun(fun,x,[2,3])

  evaluates fun at the array x and returns an array.

v1 Dec. 2011
	"""	
	y=numpy.zeros_like(x)
	
	for i in range(x.size):
		# Array consisting of [x[i], par1, par2, ...]
		p=numpy.concatenate(([x[i]],par))
		
		# Function evaluated
		y[i]=fun(p)
		
	return y







def savepartial(x,y,z,obsx,obsy,obsz,outfile):
	"""
Exports data file for partial correlation analysis with cens_tau.f.
cens_tau.f quantifies the correlation between X and Y eliminating the
effect of a third variable Z. I patched the Fortran code available from
http://astrostatistics.psu.edu/statcodes/sc_regression.html.

Method arguments:
x,y,z = arrays with data for partial correlation
obs? = arrays of integers. 1 if there is a genuine measurement
available and 0 if there is only an upper limit i.e. censored data.

In the case of this study, X=Pjet, Y=Lgamma, Z=distance.

The structure of the resulting datafile is:
	logPjet detected? logLgamma detected? logDist detected?
where the distance is in Mpc. 

Example:

>>> agngrb.exportpartial(all.kp,all.lg,log10(all.d),ones_like(all.kp),ones_like(all.kp),ones_like(all.kp),'par
tialdata.dat')

v1 Sep. 2011
	"""	
	numpy.savetxt(outfile,numpy.transpose((x,obsx,y,obsy,z,obsz)),fmt='%10.4f %i %10.4f %i %10.4f %i')






	

def log2lin(dlogyl,dlogyu,logy):
	"""
From a given uncertainty in log-space (dex) and the value of y, calculates the
error in linear space.

Returns a sequence with the lower and upper arrays with errors.
	"""
	# Upper error bar
	dyu=10**logy*(10.**dlogyu-1.)

	# Lower error bar
	dyl=-10**logy*(10.**-dlogyl-1.)

	return dyl, dyu





def lin2log(dyl,dyu,logy):
	"""
From a given uncertainty in linear space and the value of y, calculates the
error in log space.

Returns a sequence with the lower and upper arrays with errors.
	"""
	# Upper error bar
	dlogyu=-logy+numpy.log10(10.**logy+dyu)

	# Lower error bar
	dlogyl=+logy-numpy.log10(10.**logy-dyl)

	return dlogyl, dlogyu
	
	
	


def whichbces(bces):
	"""
Given the 'bces' string selector, returns an integer which tells the 
location of the BCES fitting results in the arrays returned by
the bces* methods.
	"""
	# Selects the appropriate BCES fitting method
	import sys
	
	if bces=='ort':
		i=3
	elif bces=='y|x':
		i=0
	elif bces=='x|y':
		i=1
	elif bces=='bis':
		i=2	
	else:
		sys.exit("Invalid BCES method selected! Please select bis, ort, y|x or x|y.")	

	return i
	




def mathcontour(x,y,errx,erry,cov):
	"""
Suppose I want to draw the error ellipses for two parameters 'x' and 'y' 
with 1 s.d. uncertainties and the covariance between these uncertainties. 
I have a mathematica notebook that does that: 'chisq contour plot.nb'.

This method takes the values 'x,y,errx,erry,cov' and outputs Mathematica
code that I can just copy and paste in the appropriate notebook in order 
to draw the ellipses.
	"""
	print("x0=",x,";")
	print("y0=",y,";")
	print("\[Sigma]x=",errx,";")
	print("\[Sigma]y=",erry,";")
	print("\[Sigma]xy=",cov,";")






# Methods related to datetime tuples
# ====================================
#
def convertyear(y):
	"""
Converts from decimal year to a python datetime tuple.

>>> convertyear(2012.9)

returns datetime.datetime(2012, 11, 24, 12, 0, 0, 3).

:param y: a float or array of floats
:returns: a datetime structure (or list) with the date corresponding to the input float or array

Reference: http://stackoverflow.com/questions/19305991/convert-fractional-years-to-a-real-date-in-python
	"""
	import datetime

	if numpy.size(y)==1:	# if input is float
		year = int(y)
		d = datetime.timedelta(days=(y - year)*365)
		day_one = datetime.datetime(year,1,1)
		date = d + day_one
	else:	# if input is list/array
		date=[]

		for i, yi in enumerate(y): 
			year = int(yi)
			d = datetime.timedelta(days=(yi - year)*365)
			day_one = datetime.datetime(year,1,1)
			date.append(d + day_one)

	return date


def string2year(s):
	"""
Converts from a string in the format DDMMYYYY to a python datetime tuple.

>>> string2year('28122014')

returns datetime.datetime(2014, 12, 22, 0, 0).

:param y: a string or list of strings
:returns: a datetime structure (or list) with the date corresponding to the input float or array

Reference: https://docs.python.org/2/library/datetime.html#strftime-strptime-behavior
	"""
	import datetime

	if numpy.size(s)==1:	# if input is a string
		date=datetime.datetime.strptime(str(s),'%d%m%Y')
	else:	# if input is list/array
		date=[]

		for i, si in enumerate(s): 
			date.append(datetime.datetime.strptime(str(si),'%d%m%Y'))

	return date



def date2dec(date):
	"""
Convert a python datetime tuple to decimal year.

Inspired on http://stackoverflow.com/a/6451892/793218.
	"""
	import datetime
	import time

	def sinceEpoch(date): # returns seconds since epoch
		return time.mktime(date.timetuple())

	def getyear(date): # returns decimal year for a datetime tuple
		year = date.year
		startOfThisYear = datetime.datetime(year=year, month=1, day=1)
		startOfNextYear = datetime.datetime(year=year+1, month=1, day=1)
		yearElapsed = sinceEpoch(date) - sinceEpoch(startOfThisYear)
		yearDuration = sinceEpoch(startOfNextYear) - sinceEpoch(startOfThisYear)
		fraction = yearElapsed/yearDuration
		return date.year + fraction
	
	if numpy.size(date)==1:
		year=getyear(date)
	else:
		year=[]
		for datei in date:
			year.append(getyear(datei))

	return numpy.array(year)











def runsave(cmd,log):
	"""
Executes command cmd and saves its standard output as log
	"""
	import subprocess
	
	# executes command
	p = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	out,err=p.communicate()

	# saves output in a diagnostic file
	text=open(log,"w")
	text.write(str(out))
	text.close()



def scinotation(x,n=2):
	"""
Displays a number in scientific notation.

:param x: number
:param n: number of significant digits to display
	"""
	import decimal
	s='%.'+str(n)+'E' % decimal.Decimal(str(x))

	return s






def readmodel(file,m=1e8):
    """
Auxiliary method to rescale the BHBH simulation time series from Gold+14 and 
compare with the data.
::

    tphys12, t12, y12 = readmodel('Gold paper/1,2nc mdot bin.csv')

:returns: time in years, time in code units, signal
    """
    from . import astro, dsp
    import astropy.io.ascii as ascii
    import scipy.signal

    data = ascii.read(file)
    tmod,ymod=data['col1'],data['col2']

    # cleans TS (remove duplicate times, regular dt for CWT later)
    tmod,ymod=dsp.uneven2even(tmod,ymod)
    
    # detrends data
    ymoddet=scipy.signal.detrend(ymod)
    
    # physical times in years
    const=astro.Constants()
    tphys=tmod*1000*const.G*m*const.solarmass/const.c**3/const.year
    
    return tphys, tmod, ymoddet




def adjustmodel(file,stretch,translate,obs):
    """
Auxiliary method to read and normalize the BHBH simulation TS from Gold+14 
to the observations. 
::

    tmod,smod=adjustmodel('/Users/nemmen/work/projects/fermi/ngc1275/Gold paper/1,2nc mdot bin.csv',5.6,2008.6,y)

obs is the data you want to normalize the model to.

:returns: time in years, time in code units, signal
    """
    from . import lsd
    import astropy.io.ascii as ascii
    import scipy.signal

    data = ascii.read(file)
    tmod,ymod=data['col1'],data['col2']

    # detrends and normalize data
    ymoddet=scipy.signal.detrend(ymod)
    ymod=lsd.norm(ymoddet,scipy.signal.detrend(obs))
    
    # physical times (scaled to data)
    tphys=(tmod-tmod[0])*stretch+translate
    
    # BH mass implied by the scaling, in solar masses
    # Careful with units: CGS then converted to Msun
    mass=1e-3*4.04e38*(tphys[-1]-tphys[0])/(tmod[-1]-tmod[0])*31556926./1.99e33
    print('M = ',mass,' Msun')
    
    return tphys, ymod







def paperimpact(citations,dt,impactfactor):
	"""
Given a journal `impactfactor` and a given time interval `dt`, this
computes the expected number of citations for a given paper 
published in that journal over `dt`. It then gives the ratio of your
paper's citation over the predicted value, to give you an idea
whether your paper is above or below average.

:param citations: number of citations for your paper
:param dt: time since your paper was published in years
:param impactfactor: impact factor for the journal where the paper was published
	"""
	print("Journal expected number of citations =",dt*impactfactor)

	print("Actual citations/expected citations",citations/(dt*impactfactor))




def findPATH(filename,envVar="PYTHONPATH"):
	"""
Given a PATH or PYTHONPATH environment variable, find the full path of a file 
among different options. From https://stackoverflow.com/a/1124851/793218

:param filename: file for which full path is desired
:param envVar: environment variable with path to be searched
:returns: string with full path to file

Example:

>>> fullpath=findPATH("fastregrid.cl")
	"""
	import os

	for p in os.environ[envVar].split(":"):
	    for r,d,f in os.walk(p):
	        for files in f:
	             if files == filename:
	                 return os.path.join(r,files)




"""
Signal processing
===================

Mostly time series.
"""


import numpy
import pylab
import scipy.signal




def peaks(y,x=None,what=0,**args):
	"""
Detects the peaks in the time series given by Y (and X if provided).

:param x,y: time series input arrays
:param what: select what you want -- max/0 or min/1 peaks returned
:returns: xmax,ymax -- arrays with max peaks in the data.
	"""
	from . import peakdetect	# py 3
	#import peakdetect
	
	peaks=peakdetect.peakdetect(y,x,**args)

	if what==0 or what=="max":
		return zip(*peaks[0])
	else:
		return zip(*peaks[1])





def smooth(x,window_len=11,window='hanning'):
	"""
	Smooth the data using a window with requested size.
	Copied from http://wiki.scipy.org/Cookbook/SignalSmooth
	
	This method is based on the convolution of a scaled window with the signal.
	The signal is prepared by introducing reflected copies of the signal 
	(with the window size) in both ends so that transient parts are minimized
	in the begining and end part of the output signal.
	
	:param x: the input signal 
	:param window_len: the dimension of the smoothing window; should be an odd integer
	:param window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'	flat window will produce a moving average smoothing.

	:returns: the smoothed signal
	    
	Example

	>>> t=linspace(-2,2,0.1)
	>>> x=sin(t)+randn(len(t))*0.1
	>>> y=smooth(x)
	
	.. seealso:: numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve, scipy.signal.lfilter
	
	.. note:: length(output) != length(input), to correct this: return y[(window_len/2-1):-(window_len/2)] instead of just y.

	.. todo:: the window parameter could be the window itself if an array instead of a string
	""" 
	 
	if x.ndim != 1:
	    raise ValueError("smooth only accepts 1 dimension arrays.")

	if x.size < window_len:
	    raise ValueError("Input vector needs to be bigger than window size.")
	    
	if window_len<3:
	    return x
		
	if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
	    raise ValueError("Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'")
	
	s=numpy.r_[x[window_len-1:0:-1],x,x[-1:-window_len:-1]]
	#print(len(s))
	if window == 'flat': #moving average
	    w=numpy.ones(window_len,'d')
	else:
	    w=eval('numpy.'+window+'(window_len)')
	
	y=numpy.convolve(w/w.sum(),s,mode='valid')
	return y    




def smoothxy(x,y,*arg,**args):
	"""
:param x: "time" in the time series
:param y: y variable
:returns: smoothed y and corresponding x arrays
	"""
	return smooth(x,*arg,**args), smooth(y,*arg,**args)




def varsig(f,df):
	"""
Quantifies significance of variability of time series, taking into 
account the uncertainties.

Given the time series signal *y* and the corresponding uncertainties
:math:`\sigma y`, the test statistics *T* is defined as 

.. math:: T \equiv \\frac{y_{i+1}-y_i}{\sigma y_i}.

*T* will give the significance of variability at each point in time,
in standard deviations of the preceding noise point.

**Usage:**

Computes TS and plots time series, highlighting the 1sigma variability
interval in gray:

>>> ts=varsig(flux,errflux)
>>> step(t,q,'o-')
>>> fill_between(t, -2*ones_like(tdec), 2*ones_like(tdec), alpha=0.3, facecolor='gray')

.. todo:: generalize to arbitrary windows of time
.. todo:: propagate uncertainty 
	"""
	q=numpy.zeros_like(f)
	for i,fi in enumerate(f):
		if i>0:
			dy=fi-f[i-1]
			q[i]=dy/df[i-1]

	return q





def ls(t,z,plot=False):
    """
Computes and plot Lomb-Scargle periodogram for a given timeseries. 
Returns arrays with periods and LS spectral power.

>>> p,power=l.ls(time,values,plot=False)

:returns: periods, spectral power
    """
    import scipy.signal

    tbin = t[1] - t[0] # time bin
    f = numpy.linspace(0.001, 3./tbin, 10000)
    z = scipy.signal.detrend(z)

    # periodogram
    pgram = scipy.signal.lombscargle(t,z,f)

    # Lomb-Scargle spectral power
    period=1./(f/(2*numpy.pi))
    spower=(numpy.sqrt(4.*(pgram/len(t)))/numpy.mean(numpy.sqrt(4.*(pgram/len(t)))))**2 
    
    # most significant periodicity
    #i=spower.argmax() # finds index of maximum power
    #pbest=period[i]
    #spmax=spower[i]
    #print("Spectral peak found at t="+str(pbest)+", "+str(spmax))

    # plot
    if plot==True:
        #pylab.figure(figsize=(16,8))
        pylab.title("Lomb-Scargle ")
        pylab.ylabel('Spectral Power')
        pylab.xlabel('Period [days]')
        pylab.xscale('log')
        pylab.plot(period, spower)

    return period,spower


def ls_spectra(t,var,n=200,thres=0.1,smooth=0):
    """
Computes Lomb-Scargle power spectrum, find peaks, produces arrays for
plotting images showing the spectral lines.

:param t: array of times
:param var: array of signal
:param n: number of vertical elements in image that will be created showing spectral lines
:param thres: threshold parameter for finding peaks in time series
:param smooth: number of points in the smoothing window. If 0, no smoothing

Usage: ``N,P,l,pp,power=ls_spectra(t,y,thres=0.3)``

Returns the following variables:

- ``N``: 2d number of vertical elements for plotting the spectra
- ``P``: 2d periods
- ``l``: 2d power spectra for plotting images
- ``pp``: periods corresponding to peaks (peak period) in power spectrum
- ``power``: peaks in power spectrum
    """
    import peakutils 

    p,power=ls(t,var)

    # smooth periodogram if requested
    if (smooth!=0): 
        p,power=smoothxy(p,power,smooth)

    T, P = numpy.meshgrid(range(n), p)
    
    # spectral lines
    lines = numpy.ones([1, n]) * power[:, None]
    
    # peaks
    ipeak = peakutils.indexes(power,thres=thres)
        
    return T.T, P.T, lines.T, p[ipeak], power[ipeak]






def error_resampler(errors):
    """
For use with ``pandas``.

Method for performing the proper ``mean`` resampling of the *uncertainties* (error bars) 
in the time series with ``pandas``. Note that doing a simple resampling 
will fail to propagate uncertainties, since error in the mean goes as 

.. math:: \sigma=\sqrt{\Sigma_n \sigma_n^2}

Example: Resamples the errors with 30 day averages:
::

    # df['errflux'] has the 1sigma uncertainties
    err=df['errflux'].resample('30d').apply(nmmn.dsp.error_resampler) 

    # plot y-values (df['flux']) with errors (err)
    df['flux'].resample('30d').mean().plot(yerr=err)
    """
    err=errors**2

    return numpy.sqrt(err.sum())/err.size



def sumts(t1,t2,y1,y2,n=1):
    """
Given two time series, this method returns their sum.

n = number of points in the final TS, as a multiple of the sum of points
in both TS.

Steps:

1. defines a uniform t-array with the number of total elements of the two arrays
2. interpolates both TS in this new t-array
3. sums their values
    """
    # new number of elements
    points=int(n*(t1.size+t2.size)) 
    # lower and upper ranges
    tmin=numpy.min(numpy.hstack((t1,t2)))
    tmax=numpy.max(numpy.hstack((t1,t2)))

    # Defines the new array of interpolated times    
    tnew=numpy.linspace(tmin,tmax,points)
    # interpolates the two TS
    y1new=numpy.interp(tnew,t1,y1)
    y2new=numpy.interp(tnew,t2,y2)
    # sums the two TS
    ynew=y1new+y2new

    return tnew,ynew



def dt(t):
    """
Computes difference between consecutive points in the time series.

:param t: input times
:param y: input y-values
    """
    dtt=[]
    for i in range(t.size):
        if (i>0): 
            dtt.append(t[i]-t[i-1])

    return dtt


def uneven2even(t,y):
    """
Given an uneven timeseries (TS) with multiple values defined at the same 
time, this method will convert it into an evenly sampled
timeseries with a dt as close as possible to the actual dt, removing
duplicate-t values.

Example: suppose you want to compute the CWT for an unevenly sampled
time series. Since the CWT does not support uneven TS, you can first
call this method to regularize your TS and then perform the TS.

Algorithm:

- REMOVE DUPLICATE times
- CREATE REGULAR GRID USING BEST DT
- INTERPOLATE NEW TS ON PREVIOUS ONE 

:param t: input times
:param y: input value
    """
    import scipy.interpolate

    # remove items with same t
    # ==========================
    # gets successive dt for all points
    dtarr=[] 
    idel=[] # list of indexes for elements that will be removed
    for i in range(t.size):
        if (i>0):
            dt=t[i]-t[i-1]
            if (dt==0): # if they have dt==0, store their index and later remove them
                idel.append(i)
            else:
                dtarr.append(dt) # stores only dt!=0

    # Find out optimal value of dt for the new TS
    dt=numpy.mean(dtarr)

    # Removes elements with same t
    tuniq=numpy.delete(t,idel)
    yuniq=numpy.delete(y,idel)

    # Does linear interpolation on new TS
    # ======================================
    # new regular grid, as close as possible to original one
    tnew=numpy.arange(t[0],t[-1],dt)

    # interpolation
    f = scipy.interpolate.interp1d(tuniq, yuniq)
    ynew = f(tnew)

    return tnew,ynew





# Wavelet methods
# ==================


def cwt(t,sig):

    """
Given the time and flux (sig), this method computes a continuous 
wavelet transform (CWT).

.. warning:: Deprecated.
    """
    import matplotlib
    import matplotlib.pyplot as plt
    import obspy.signal.tf_misfit
    import scipy.signal


    tbin = t[1] - t[0]
    freqLC = numpy.fft.fftfreq(len(t),d=tbin)
    fluxo = sig
    
    sig = scipy.signal.detrend(sig)
    fig = plt.figure()
    f_min = 5./t.max()
    f_max = 1000*freqLC.max()     #so calcula ate onde vai a FFT. Porque nao faz sentido tentar encontrar frequencias alem da resolucao temporal
    gs0 = matplotlib.gridspec.GridSpec(2, 2,hspace=0,wspace=0)   #GridSpec specifies the geometry of the grid that a subplot will be placed. 
                                                      #The number of rows and number of columns of the grid need to be set (here: 2 x 2).
                                                      #The horizontal and vertical space between graphics can be adjusted in "hspace" and "wspace"
    ax = fig.add_subplot(gs0[0])
    dt = 0.001*tbin
    CWT = obspy.signal.tf_misfit.cwt(sig,dt,6,f_min,f_max,nf=1000)
    x, y = numpy.meshgrid(t, numpy.logspace(numpy.log10(f_min), numpy.log10(f_max), CWT.shape[0]))
    NormCWT = (numpy.abs(CWT)**2)/numpy.var(fluxo)


    return x, 1000./y, NormCWT




def reconstruct(wa,i=None):
    """
Method to reconstruct a time series (TS) signal based on its CWT. 

:param wa: wavelet object generated with `wavelets` module
:param i: index array containing only the elements that will be excluded from the signal reconstruction. i.e. the elements indicated by ``i`` will zeroed in the CWT complex array.
:returns: full reconstructed TS, reconstructed CWT power array, reconstructed CWT complex array, detrended reconstructed TS

Examples: 

1. Compute the CWT for TS in ``var``:
::

    import wavelets

    # compute CWT
    wa = wavelets.WaveletAnalysis(var, dt=dt)

    # time and period 2D arrays
    T, S = numpy.meshgrid(wa.time, wa.scales)
  
2. Reconstruct the signal, throwing out all periods with values between 1000 and 2000:
::

    j=where((S<1000) | (S>2000))
    recdet,rec=nmmn.dsp.reconstruct(wa,j)

3. Plots the reconstructed signal along with the data:
::

    subplot(2,1,1)
    plot(t,flux,label='original')
    plot(t,rec,label='reconstructed signal',lw=5)

    subplot(2,1,2)
    plot(t,recdet,'r')
    title('Pure reconstructed signal')

    """
    wavecut=wa.wavelet_transform
    if i is not None: wavecut[i]=0
    
    # reconstructed signal
    rec=wa.reconstruction(wave=wavecut)
    
    # find the linear trend on the signal
    import scipy.stats
    a, b, r, p, err = scipy.stats.linregress(wa.time,wa.data)

    # detrend the reconstructed signal
    import scipy.signal
    recdet=scipy.signal.detrend(rec)
    
    return recdet+a*wa.time+b,numpy.abs(wavecut)**2,wavecut,recdet



def reconstruct_period(wa,P1,P2):
    """
Returns reconstructed detrended signal from CWT, considering only the interval
between the two given periods.
    """
    T, P = numpy.meshgrid(wa.time, wa.scales)

    i=numpy.where((P<P1) | (P>P2))
    xrec,powerrec,cwtrec,xrecdet=reconstruct(wa,i)
    return xrecdet




def cwt_spectra(t,var,dj=0.01,n=200,thres=0.1):
    """
Computes CWT power spectrum, find peaks, produces arrays for
plotting images showing the spectral lines. Note that the CWT power spectrum
is an average from the CWT power array. Therefore, they are a smoothed out 
version of a Fourier spectrum.

:param t: array of times
:param var: array of signal
:param n: number of vertical elements in image that will be created showing spectral lines
:param thres: threshold parameter for finding peaks in time series

Usage: ``N,P,l,pp,power=cwt_spectra(t,y,thres=0.3)``

Returns the following variables:

- ``N``: 2d number of vertical elements for plotting the spectra
- ``P``: 2d periods
- ``l``: 2d power spectra for plotting images
- ``pp``: periods corresponding to peaks (peak period) in power spectrum
- ``power``: peaks in CWT power spectrum
    """    
    import peakutils, wavelets

    dt=t[1]-t[0] # bin size
    wa = wavelets.WaveletAnalysis(var, dt=dt,dj=dj)
    T, P = numpy.meshgrid(range(n), wa.scales)
    cwtpower=wa.wavelet_power
    
    # spectral lines
    z=cwtpower.mean(axis=1)
    lines = numpy.ones([1, n]) * z[:, None]
    
    # peaks
    ipeak = peakutils.indexes(z,thres=thres)
        
    return T.T, P.T, lines.T, wa.scales[ipeak], z[ipeak]





"""
Fancy plots
==============
"""

import numpy
from matplotlib import pylab
from nmmn import sed

def plot(spec):
	"""
Returns the plot of a grmonty spectrum as a pyplot object or plot it on 
the screen
	
:param param: grmonty spectrum file
	"""

	s = sed.SED()
	s.grmonty(spec)
	pylab.plot(s.lognu, s.ll)
	pylab.show()
	

	

def onehist(x,xlabel='',fontsize=12):
	""" 
Script that plots the histogram of x with the corresponding xlabel. 
	"""

	pylab.clf()
	pylab.rcParams.update({'font.size': fontsize})
	pylab.hist(x,histtype='stepfilled')
	pylab.legend()
	#### Change the X-axis appropriately ####
	pylab.xlabel(xlabel)
	pylab.ylabel('Number')
	pylab.draw()
	pylab.show()





def twohists(x1,x2,xmin,xmax,range=None,x1leg='$x_1$',x2leg='$x_2$',xlabel='',fig=1,sharey=False,fontsize=12,bins1=10,bins2=10):
	"""
Script that plots two histograms of quantities x1 and x2
sharing the same X-axis.

:param x1,x2: arrays with data to be plotted
:param xmin,xmax: lower and upper range of plotted values, will be used to set a consistent x-range
	for both histograms.
:param x1leg, x2leg: legends for each histogram	
:param xlabel: self-explanatory.
:param bins1,bins2: number of bins in each histogram
:param fig: which plot window should I use?
:param range: in the form (xmin,xmax), same as range argument for hist and applied to both
	histograms.

Inspired by `Scipy <http://www.scipy.org/Cookbook/Matplotlib/Multiple_Subplots_with_One_Axis_Label>`_.
	"""

	pylab.rcParams.update({'font.size': fontsize})
	fig=pylab.figure(fig)
	pylab.clf()
	
	a=fig.add_subplot(2,1,1)
	if sharey==True:
		b=fig.add_subplot(2,1,2, sharex=a, sharey=a)
	else:
		b=fig.add_subplot(2,1,2, sharex=a)
	
	a.hist(x1,bins1,label=x1leg,color='b',histtype='stepfilled',range=range)
	a.legend(loc='best',frameon=False)
	a.set_xlim(xmin,xmax)
	
	b.hist(x2,bins2,label=x2leg,color='r',histtype='stepfilled',range=range)
	b.legend(loc='best',frameon=False)
	
	pylab.setp(a.get_xticklabels(), visible=False)

	b.set_xlabel(xlabel)
	b.set_ylabel('Number',verticalalignment='bottom')
	pylab.minorticks_on()
	pylab.subplots_adjust(hspace=0.15)
	pylab.draw()
	pylab.show()





def threehists(x1,x2,x3,xmin,xmax,x1leg='$x_1$',x2leg='$x_2$',x3leg='$x_3$',xlabel='',fig=1,sharey=False,fontsize=12):
	"""
Script that plots three histograms of quantities x1, x2 and x3 
sharing the same X-axis.

Arguments:
- x1,x2,x3: arrays with data to be plotted
- xmin,xmax: lower and upper range of plotted values, will be used to set a consistent x-range for both histograms.
- x1leg, x2leg, x3leg: legends for each histogram	
- xlabel: self-explanatory.
- sharey: sharing the Y-axis among the histograms?
- fig: which plot window should I use?

Example:
x1=Lbol(AD), x2=Lbol(JD), x3=Lbol(EHF10)

>>> threehists(x1,x2,x3,38,44,'AD','JD','EHF10','$\log L_{\\rm bol}$ (erg s$^{-1}$)',sharey=True)

Inspired by `Scipy <http://www.scipy.org/Cookbook/Matplotlib/Multiple_Subplots_with_One_Axis_Label>`_.
	"""
	pylab.rcParams.update({'font.size': fontsize})
	fig=pylab.figure(fig)
	pylab.clf()
	
	a=fig.add_subplot(3,1,1)
	if sharey==True:
		b=fig.add_subplot(3,1,2, sharex=a, sharey=a)
		c=fig.add_subplot(3,1,3, sharex=a, sharey=a)
	else:
		b=fig.add_subplot(3,1,2, sharex=a)
		c=fig.add_subplot(3,1,3, sharex=a)		
	
	a.hist(x1,label=x1leg,color='b',histtype='stepfilled')
	a.legend(loc='best',frameon=False)
	a.set_xlim(xmin,xmax)
	
	b.hist(x2,label=x2leg,color='r',histtype='stepfilled')
	b.legend(loc='best',frameon=False)

	c.hist(x3,label=x3leg,color='y',histtype='stepfilled')
	c.legend(loc='best',frameon=False)
	
	pylab.setp(a.get_xticklabels(), visible=False)
	pylab.setp(b.get_xticklabels(), visible=False)

	c.set_xlabel(xlabel)
	b.set_ylabel('Number')
	pylab.minorticks_on()
	pylab.subplots_adjust(hspace=0.15)
	pylab.draw()
	pylab.show()
	
	
	




def fourhists(x1,x2,x3,x4,xmin,xmax,x1leg='$x_1$',x2leg='$x_2$',x3leg='$x_3$',x4leg='$x_3$',xlabel='',fig=1,sharey=False,fontsize=12,bins1=10,bins2=10,bins3=10,bins4=10,line1=None,line2=None,line3=None,line4=None,line1b=None,line2b=None,line3b=None,line4b=None,loc='best'):
	"""
Script that plots four histograms of quantities x1, x2, x3 and x4
sharing the same X-axis.

Arguments:

- x1,x2,x3,x4: arrays with data to be plotted
- xmin,xmax: lower and upper range of plotted values, will be used to set a consistent x-range
or both histograms.
- x1leg, x2leg, x3leg, x4leg: legends for each histogram	
- xlabel: self-explanatory.
- sharey: sharing the Y-axis among the histograms?
- bins1,bins2,...: number of bins in each histogram
- fig: which plot window should I use?
- line?: draws vertical solid lines at the positions indicated in each panel
- line?b: draws vertical dashed lines at the positions indicated in each panel

Inspired by `Scipy <http://www.scipy.org/Cookbook/Matplotlib/Multiple_Subplots_with_One_Axis_Label>`_.
	"""
	pylab.rcParams.update({'font.size': fontsize})
	fig=pylab.figure(fig)
	pylab.clf()
	
	a=fig.add_subplot(4,1,1)
	if sharey==True:
		b=fig.add_subplot(4,1,2, sharex=a, sharey=a)
		c=fig.add_subplot(4,1,3, sharex=a, sharey=a)
		d=fig.add_subplot(4,1,4, sharex=a, sharey=a)
	else:
		b=fig.add_subplot(4,1,2, sharex=a)
		c=fig.add_subplot(4,1,3, sharex=a)		
		d=fig.add_subplot(4,1,4, sharex=a)
	
	def vline(hist,value,linestyle='k'):
		"""Draw vertical line"""
		yax=hist.set_ylim()
		hist.plot([value,value],[yax[0],yax[1]],linestyle,linewidth=2)

	a.hist(x1,bins1,label=x1leg,color='b',histtype='stepfilled')
	a.legend(loc=loc,frameon=False)
	a.set_xlim(xmin,xmax)
	if line1!=None: vline(a,line1)
	if line1b!=None: vline(a,line1b,'k--')
	
	b.hist(x2,bins2,label=x2leg,color='r',histtype='stepfilled')
	b.legend(loc=loc,frameon=False)
	if line2!=None: vline(b,line2)
	if line2b!=None: vline(b,line2b,'k--')

	c.hist(x3,bins3,label=x3leg,color='y',histtype='stepfilled')
	c.legend(loc=loc,frameon=False)
	if line3!=None: vline(c,line3)
	if line3b!=None: vline(c,line3b,'k--')

	d.hist(x4,bins4,label=x4leg,color='g',histtype='stepfilled')
	d.legend(loc=loc,frameon=False)
	if line4!=None: vline(d,line4)
	if line4b!=None: vline(d,line4b,'k--')
	
	pylab.setp(a.get_xticklabels(), visible=False)
	pylab.setp(b.get_xticklabels(), visible=False)
	pylab.setp(c.get_xticklabels(), visible=False)

	d.set_xlabel(xlabel)
	c.set_ylabel('Number')
	pylab.minorticks_on()
	pylab.subplots_adjust(hspace=0.15)
	pylab.draw()
	pylab.show()






def fourcumplot(x1,x2,x3,x4,xmin,xmax,x1leg='$x_1$',x2leg='$x_2$',x3leg='$x_3$',x4leg='$x_3$',xlabel='',ylabel='$N(x>x\')$',fig=1,sharey=False,fontsize=12,bins1=50,bins2=50,bins3=50,bins4=50):
	"""
Script that plots the cumulative histograms of four variables x1, x2, x3 and x4
sharing the same X-axis. For each bin, Y is the fraction of the sample 
with values above X.

Arguments:

- x1,x2,x3,x4: arrays with data to be plotted
- xmin,xmax: lower and upper range of plotted values, will be used to set a consistent x-range
for both histograms.
- x1leg, x2leg, x3leg, x4leg: legends for each histogram	
- xlabel: self-explanatory.
- sharey: sharing the Y-axis among the histograms?
- bins1,bins2,...: number of bins in each histogram
- fig: which plot window should I use?

Inspired by `Scipy <http://www.scipy.org/Cookbook/Matplotlib/Multiple_Subplots_with_One_Axis_Label>`_.

v1 Jun. 2012: inherited from fourhists.
	"""
	pylab.rcParams.update({'font.size': fontsize})
	fig=pylab.figure(fig)
	pylab.clf()
	
	a=fig.add_subplot(4,1,1)
	if sharey==True:
		b=fig.add_subplot(4,1,2, sharex=a, sharey=a)
		c=fig.add_subplot(4,1,3, sharex=a, sharey=a)
		d=fig.add_subplot(4,1,4, sharex=a, sharey=a)
	else:
		b=fig.add_subplot(4,1,2, sharex=a)
		c=fig.add_subplot(4,1,3, sharex=a)		
		d=fig.add_subplot(4,1,4, sharex=a)
	
	a.hist(x1,bins1,label=x1leg,color='b',cumulative=-True,normed=True,histtype='stepfilled')
	a.legend(loc='best',frameon=False)
	a.set_xlim(xmin,xmax)
	
	b.hist(x2,bins2,label=x2leg,color='r',cumulative=-True,normed=True,histtype='stepfilled')
	b.legend(loc='best',frameon=False)

	c.hist(x3,bins3,label=x3leg,color='y',cumulative=-True,normed=True,histtype='stepfilled')
	c.legend(loc='best',frameon=False)

	d.hist(x4,bins4,label=x4leg,color='g',cumulative=-True,normed=True,histtype='stepfilled')
	d.legend(loc='best',frameon=False)
	
	pylab.setp(a.get_xticklabels(), visible=False)
	pylab.setp(b.get_xticklabels(), visible=False)
	pylab.setp(c.get_xticklabels(), visible=False)

	d.set_xlabel(xlabel)
	c.set_ylabel(ylabel)
	pylab.minorticks_on()
	pylab.subplots_adjust(hspace=0.15)
	pylab.draw()
	pylab.show()







def threehistsx(x1,x2,x3,x1leg='$x_1$',x2leg='$x_2$',x3leg='$x_3$',fig=1,fontsize=12,bins1=10,bins2=10,bins3=10):
	"""
Script that pretty-plots three histograms of quantities x1, x2 and x3.

Arguments:
:param x1,x2,x3: arrays with data to be plotted
:param x1leg, x2leg, x3leg: legends for each histogram	
:param fig: which plot window should I use?

Example:
x1=Lbol(AD), x2=Lbol(JD), x3=Lbol(EHF10)

>>> threehists(x1,x2,x3,38,44,'AD','JD','EHF10','$\log L_{\\rm bol}$ (erg s$^{-1}$)')

Inspired by http://www.scipy.org/Cookbook/Matplotlib/Multiple_Subplots_with_One_Axis_Label.
	"""
	pylab.rcParams.update({'font.size': fontsize})
	pylab.figure(fig)
	pylab.clf()
	
	pylab.subplot(3,1,1)
	pylab.hist(x1,label=x1leg,color='b',bins=bins1)
	pylab.legend(loc='best',frameon=False)

	pylab.subplot(3,1,2)
	pylab.hist(x2,label=x2leg,color='r',bins=bins2)
	pylab.legend(loc='best',frameon=False)

	pylab.subplot(3,1,3)
	pylab.hist(x3,label=x3leg,color='y',bins=bins3)
	pylab.legend(loc='best',frameon=False)

	pylab.minorticks_on()
	pylab.subplots_adjust(hspace=0.15)
	pylab.draw()
	pylab.show()

	
	

	
def fitconf(xdata,ydata,errx,erry,covxy,nboot=1000,bcesMethod='ort',linestyle='',conf=0.683,confcolor='gray',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the input data performs the BCES
	fit, get the orthogonal parameters and plot the best-fit line and
	confidence band (generated using analytical methods). I decided to put together 
	these commands in a method because I have been using them very frequently.
	
	Assumes you initialized the plot window before calling this method.
	
	Usage:

	>>> a1,b1,erra1,errb1,cov1=nemmen.fitconf(x[i],y[i],errx[i],erry[i],covxy[i],nboot,bces,linestyle='k',confcolor='LightGrey')
	
	Explanation of some arguments:

	- xplot: if provided, will compute the confidence band in the X-values provided
	with xplot
	- front: if True, then will plot the confidence band in front of the data
	points; otherwise, will plot it behind the points
	"""	
	import bces.bces
	from . import stats
	from . import misc	

	# Selects the desired BCES method
	i=misc.whichbces(bcesMethod)
		
	# Performs the BCES fit
	a,b,erra,errb,cov=bces.bces.bcesp(xdata,errx,ydata,erry,covxy,nboot)
	
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	pylab.plot(x,a[i]*x+b[i],linestyle,**args)

	fitm=numpy.array([ a[i],b[i] ])	# array with best-fit parameters
	covm=numpy.array([ (erra[i]**2,cov[i]), (cov[i],errb[i]**2) ])	# covariance matrix
	def func(x): return x[1]*x[0]+x[2]

	# Plots confidence band
	lcb,ucb,xcb=stats.confbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	if front==True:
		zorder=10
	else:
		zorder=None
	pylab.fill_between(xcb, lcb, ucb, alpha=0.3, facecolor=confcolor, zorder=zorder)
	
	return a,b,erra,errb,cov





def fitconfmc(xdata,ydata,errx,erry,covxy,nboot=1000,bces='ort',linestyle='',conf=1.,confcolor='gray',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the input data performs the BCES
	fit, get the orthogonal parameters and plot the best-fit line and
	confidence band (generated using MC). I decided to put together these 
	commands in a method because I have been using them very frequently.
	
	Assumes you initialized the plot window before calling this method.
	This method is more stable than fitconf, which is plagued with numerical 
	instabilities when computing the gradient.
	
	Usage:

	>>> a1,b1,erra1,errb1,cov1=nemmen.fitconf(x[i],y[i],errx[i],erry[i],covxy[i],nboot,bces,linestyle='k',confcolor='LightGrey')
	
	Explanation of some arguments:
	- xplot: if provided, will compute the confidence band in the X-values provided
	with xplot
	- front: if True, then will plot the confidence band in front of the data
	points; otherwise, will plot it behind the points
	- conf: size of confidence band to be plotted in standard deviations
	"""	
	# Selects the desired BCES method
	i=whichbces(bces)
		
	# Performs the BCES fit
	a,b,erra,errb,cov=bcesp(xdata,errx,ydata,erry,covxy,nboot)
	
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	pylab.plot(x,a[i]*x+b[i],linestyle,**args)

	fitm=numpy.array([ a[i],b[i] ])	# array with best-fit parameters
	covm=numpy.array([ (erra[i]**2,cov[i]), (cov[i],errb[i]**2) ])	# covariance matrix

	# Plots confidence band
	lcb,ucb,y=confbandmc(x,fitm,covm,10000,conf)
	if front==True:
		zorder=10
	else:
		zorder=None
	pylab.fill_between(x, lcb, ucb, alpha=0.3, facecolor=confcolor, zorder=zorder)
	
	return a,b,erra,errb,cov










def plotlinfit(xdata,ydata,a,b,erra,errb,cov,linestyle='',conf=0.683,confcolor='gray',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the output data from a linear regression
	method (for example, bayeslin.pro, the Bayesian linear regression method 
	of Kelly (2007)), it plots the fits and the confidence bands.
	The input is:
	X, Y, slope (A), errA, intercept (B), errB and cov(A,B)
	
	Assumes you initialized the plot window before calling this method.
	
	Usage:

	>>> nemmen.plotlinfit(x,y,a,b,erra,errb,covab,linestyle='k',confcolor='LightGrey')
	
	Explanation of some arguments:
	- xplot: if provided, will compute the confidence band in the X-values provided
	with xplot
	- front: if True, then will plot the confidence band in front of the data
	points; otherwise, will plot it behind the points
	"""			
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	pylab.plot(x,a*x+b,linestyle,**args)

	fitm=numpy.array([ a,b ])	# array with best-fit parameters
	covm=numpy.array([ (erra**2,cov), (cov,errb**2) ])	# covariance matrix
	def func(x): return x[1]*x[0]+x[2]

	# Plots confidence band
	lcb,ucb,xcb=confbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	if front==True:
		zorder=10
	else:
		zorder=None
	pylab.fill_between(xcb, lcb, ucb, alpha=0.3, facecolor=confcolor, zorder=zorder)
	




def jh(xdata,ydata,errx,erry,covxy,nboot=1000,bces='ort',linestyle='',conf=0.683,confcolor='gray',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the input data performs the BCES
	fit, get the orthogonal parameters, best-fit line and
	confidence band. Then returns the points corresponding to the line and
	confidence band. 

	I wrote this for the John Hunter plotting contest, in order to simplify
	my AGN-GRB plot. Inherited from method fitconf.
	
	Usage:

	>>> x,y,lcb,ucb=nemmen.fitconf(x[i],y[i],errx[i],erry[i],covxy[i],nboot,bces,linestyle='k',confcolor='LightGrey')

	where y are the line points, lcb and ucb are the lower and upper confidence band
	points.

	:param xplot: if provided, will compute the confidence band in the X-values provided
	  with xplot
	:param front: if True, then will plot the confidence band in front of the data
	  points; otherwise, will plot it behind the points
	"""	
	# Selects the desired BCES method
	i=whichbces(bces)
		
	# Performs the BCES fit
	a,b,erra,errb,cov=bcesp(xdata,errx,ydata,erry,covxy,nboot)
	
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	y=a[i]*x+b[i]

	fitm=numpy.array([ a[i],b[i] ])	# array with best-fit parameters
	covm=numpy.array([ (erra[i]**2,cov[i]), (cov[i],errb[i]**2) ])	# covariance matrix
	def func(x): return x[1]*x[0]+x[2]

	# Plots confidence band
	lcb,ucb,xcb=confbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	
	return x,y,lcb,ucb










def fitconfpred(xdata,ydata,errx,erry,covxy,nboot=1000,bces='ort',linestyle='',conf=0.68,confcolor='LightGrey',predcolor='Khaki',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the input data performs the BCES
	fit, get the orthogonal parameters and plot (i) the best-fit line,
	(ii) confidence band and (iii) prediction band. 
	
	I decided to put together these commands in a method because I have been 
	using them very frequently.
	
	Assumes you initialized the plot window before calling this method.
	
	Usage:

	>>> a1,b1,erra1,errb1,cov1=nemmen.fitconfpred(x[i],y[i],errx[i],erry[i],covxy[i],nboot,bces,linestyle='k',confcolor='LightGrey')
	"""	
	# Selects the desired BCES method
	i=whichbces(bces)
		
	# Performs the BCES fit
	a,b,erra,errb,cov=bcesp(xdata,errx,ydata,erry,covxy,nboot)
	
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	pylab.plot(x,a[i]*x+b[i],linestyle,**args)

	fitm=numpy.array([ a[i],b[i] ])	# array with best-fit parameters
	covm=numpy.array([ (erra[i]**2,cov[i]), (cov[i],errb[i]**2) ])	# covariance matrix
	def func(x): return x[1]*x[0]+x[2]
	
	if front==True:
		zorder=10
	else:
		zorder=None
	
	# Plots prediction band
	lpb,upb,xpb=predbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	pylab.fill_between(xpb, lpb, upb, facecolor=predcolor,edgecolor='', zorder=zorder)
	
	# Plots confidence band
	lcb,ucb,xcb=confbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	pylab.fill_between(xcb, lcb, ucb, facecolor=confcolor,edgecolor='', zorder=zorder)
	
	return a,b,erra,errb,cov




def fitpred(xdata,ydata,errx,erry,covxy,nboot=1000,bces='ort',linestyle='',conf=0.68,predcolor='Khaki',xplot=None,front=False,**args):
	"""
	This is a wrapper that given the input data performs the BCES
	fit, get the orthogonal parameters and plot (i) the best-fit line and
	(ii) prediction band. 
	
	I decided to put together these commands in a method because I have been 
	using them very frequently.
	
	Assumes you initialized the plot window before calling this method.
	
	Usage:

	>>> a1,b1,erra1,errb1,cov1=nemmen.fitpred(x[i],y[i],errx[i],erry[i],covxy[i],nboot,bces,linestyle='k',predcolor='LightGrey')
	"""	
	# Selects the desired BCES method
	i=whichbces(bces)
		
	# Performs the BCES fit
	a,b,erra,errb,cov=bcesp(xdata,errx,ydata,erry,covxy,nboot)
	
	# Plots best-fit
	if xplot==None:
		x=numpy.linspace(xdata.min(),xdata.max(),100)
	else:
		x=xplot
	pylab.plot(x,a[i]*x+b[i],linestyle,**args)

	fitm=numpy.array([ a[i],b[i] ])	# array with best-fit parameters
	covm=numpy.array([ (erra[i]**2,cov[i]), (cov[i],errb[i]**2) ])	# covariance matrix
	def func(x): return x[1]*x[0]+x[2]
	
	if front==True:
		zorder=10
	else:
		zorder=None
	
	# Plots prediction band
	lpb,upb,xpb=predbandnl(xdata,ydata,func,fitm,covm,2,conf,x)
	pylab.fill_between(xpb, lpb, upb, facecolor=predcolor,edgecolor='', zorder=zorder)
	
	return a,b,erra,errb,cov






def uerrorbar(ux,uy,**args):
	"""
Adaptation of pylab.errorbar to work with arrays defined using the
uncertainties package, which include the errorbars built-in.

Usage:

>>> uerrorbar(x,y,fmt='o')

will plot the points and error bars associated with the 'unumpy'
arrays x and y
	"""	
	x=unumpy.nominal_values(ux)
	y=unumpy.nominal_values(uy)
	errx=unumpy.std_devs(ux)
	erry=unumpy.std_devs(uy)
	
	pylab.errorbar(x,y,xerr=errx,yerr=erry,**args)



def text(x,y,s,**args):
	"""
Version of pylab.text that can be applied to arrays.

Usage:

>>> text(x,y,s, fontsize=10)

will plot the strings in array 's' at coordinates given by arrays
'x' and 'y'.
	"""
	for j in range(x.size):
		pylab.text(x[j],y[j],s[j], **args)



def ipyplots():
	"""
Makes sure we have exactly the same matplotlib settings as in the IPython terminal 
version. Call this from IPython notebook.

`Source <http://stackoverflow.com/questions/16905028/why-is-matplotlib-plot-produced-from-ipython-notebook-slightly-different-from-te)>`_.
	"""
	pylab.rcParams['figure.figsize']=(8.0,6.0)    #(6.0,4.0)
	pylab.rcParams['font.size']=12                #10 
	pylab.rcParams['savefig.dpi']=100             #72 
	pylab.rcParams['figure.subplot.bottom']=.1    #.125




def make_cmap(colors, position=None, bit=False):
    '''
    make_cmap takes a list of tuples which contain RGB values. The RGB
    values may either be in 8-bit [0 to 255] (in which bit must be set to
    True when called) or arithmetic [0 to 1] (default). make_cmap returns
    a cmap with equally spaced colors.
    Arrange your tuples so that the first color is the lowest value for the
    colorbar and the last is the highest.
    position contains values from 0 to 1 to dictate the location of each color.

    source: http://schubert.atmos.colostate.edu/~cslocum/custom_cmap.html
    Chris Slocum, Colorado State University
    '''
    import matplotlib as mpl
    import numpy as np
    bit_rgb = np.linspace(0,1,256)
    if position == None:
        position = np.linspace(0,1,len(colors))
    else:
        if len(position) != len(colors):
            sys.exit("position length must be the same as colors")
        elif position[0] != 0 or position[-1] != 1:
            sys.exit("position must start with 0 and end with 1")
    if bit:
        for i in range(len(colors)):
            colors[i] = (bit_rgb[colors[i][0]],
                         bit_rgb[colors[i][1]],
                         bit_rgb[colors[i][2]])
    cdict = {'red':[], 'green':[], 'blue':[]}
    for pos, color in zip(position, colors):
        cdict['red'].append((pos, color[0], color[0]))
        cdict['green'].append((pos, color[1], color[1]))
        cdict['blue'].append((pos, color[2], color[2]))

    cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,256)
    return cmap


def image(Z,xnew,ynew,my_cmap=None,aspect='equal'):
	"""
Creates pretty image. You need to specify:
	"""
	imshow(log10(Z),extent=[xnew[0],xnew[-1],ynew[0],ynew[-1]], cmap=my_cmap)
	pylab.axes().set_aspect('equal')
	colorbar()
	circle2=Circle((0,0),1,color='k')
	gca().add_artist(circle2)
	savefig('tmp.png',transparent=True,dpi=150)




def wolframcmap():
	"""
	Returns colormap that matches closely the one used by default
	for images in Wolfram Mathematica 11 (dark blue to orange).

	I spent one hour playing around to reproduce it.

	Usage:

	>>> mycmap=nmmn.plots.wolframcmap()
	>>> imshow(rho, cmap=mycmap)
	"""
	# Create a list of RGB tuples, recreates Mathematica colormap
	colors3=[(51,91,150),(111,116,143),(167,136,110),(233,167,85),(251,212,141),(255,247,190)]

	# Call the function make_cmap which returns your colormap
	return make_cmap(colors3, bit=True)





def parulacmap():
	"""
	Creates the beautiful Parula colormap which is Matlab's default.

	Usage:

	>>> mycmap=nmmn.plots.parulacmap()
	>>> imshow(rho, cmap=mycmap)

	Code taken from `here <https://github.com/BIDS/colormap/blob/master/parula.py>`_
	"""
	from matplotlib.colors import LinearSegmentedColormap

	cm_data = [[0.2081, 0.1663, 0.5292], [0.2116238095, 0.1897809524, 0.5776761905], 
	 [0.212252381, 0.2137714286, 0.6269714286], [0.2081, 0.2386, 0.6770857143], 
	 [0.1959047619, 0.2644571429, 0.7279], [0.1707285714, 0.2919380952, 
	  0.779247619], [0.1252714286, 0.3242428571, 0.8302714286], 
	 [0.0591333333, 0.3598333333, 0.8683333333], [0.0116952381, 0.3875095238, 
	  0.8819571429], [0.0059571429, 0.4086142857, 0.8828428571], 
	 [0.0165142857, 0.4266, 0.8786333333], [0.032852381, 0.4430428571, 
	  0.8719571429], [0.0498142857, 0.4585714286, 0.8640571429], 
	 [0.0629333333, 0.4736904762, 0.8554380952], [0.0722666667, 0.4886666667, 
	  0.8467], [0.0779428571, 0.5039857143, 0.8383714286], 
	 [0.079347619, 0.5200238095, 0.8311809524], [0.0749428571, 0.5375428571, 
	  0.8262714286], [0.0640571429, 0.5569857143, 0.8239571429], 
	 [0.0487714286, 0.5772238095, 0.8228285714], [0.0343428571, 0.5965809524, 
	  0.819852381], [0.0265, 0.6137, 0.8135], [0.0238904762, 0.6286619048, 
	  0.8037619048], [0.0230904762, 0.6417857143, 0.7912666667], 
	 [0.0227714286, 0.6534857143, 0.7767571429], [0.0266619048, 0.6641952381, 
	  0.7607190476], [0.0383714286, 0.6742714286, 0.743552381], 
	 [0.0589714286, 0.6837571429, 0.7253857143], 
	 [0.0843, 0.6928333333, 0.7061666667], [0.1132952381, 0.7015, 0.6858571429], 
	 [0.1452714286, 0.7097571429, 0.6646285714], [0.1801333333, 0.7176571429, 
	  0.6424333333], [0.2178285714, 0.7250428571, 0.6192619048], 
	 [0.2586428571, 0.7317142857, 0.5954285714], [0.3021714286, 0.7376047619, 
	  0.5711857143], [0.3481666667, 0.7424333333, 0.5472666667], 
	 [0.3952571429, 0.7459, 0.5244428571], [0.4420095238, 0.7480809524, 
	  0.5033142857], [0.4871238095, 0.7490619048, 0.4839761905], 
	 [0.5300285714, 0.7491142857, 0.4661142857], [0.5708571429, 0.7485190476, 
	  0.4493904762], [0.609852381, 0.7473142857, 0.4336857143], 
	 [0.6473, 0.7456, 0.4188], [0.6834190476, 0.7434761905, 0.4044333333], 
	 [0.7184095238, 0.7411333333, 0.3904761905], 
	 [0.7524857143, 0.7384, 0.3768142857], [0.7858428571, 0.7355666667, 
	  0.3632714286], [0.8185047619, 0.7327333333, 0.3497904762], 
	 [0.8506571429, 0.7299, 0.3360285714], [0.8824333333, 0.7274333333, 0.3217], 
	 [0.9139333333, 0.7257857143, 0.3062761905], [0.9449571429, 0.7261142857, 
	  0.2886428571], [0.9738952381, 0.7313952381, 0.266647619], 
	 [0.9937714286, 0.7454571429, 0.240347619], [0.9990428571, 0.7653142857, 
	  0.2164142857], [0.9955333333, 0.7860571429, 0.196652381], 
	 [0.988, 0.8066, 0.1793666667], [0.9788571429, 0.8271428571, 0.1633142857], 
	 [0.9697, 0.8481380952, 0.147452381], [0.9625857143, 0.8705142857, 0.1309], 
	 [0.9588714286, 0.8949, 0.1132428571], [0.9598238095, 0.9218333333, 
	  0.0948380952], [0.9661, 0.9514428571, 0.0755333333], 
	 [0.9763, 0.9831, 0.0538]]

	return LinearSegmentedColormap.from_list('parula', cm_data)












def jointplot(X,Y,xlabel=None,ylabel=None,binsim=40,binsh=20,contour=True):
	"""
Plots the joint distribution of posteriors for X1 and X2, including the 1D
histograms showing the median and standard deviations.

The work that went in creating this nice method is shown, step by step, in 
the ipython notebook "error contours.ipynb". Sources of inspiration:
- http://python4mpia.github.io/intro/quick-tour.html
- http://stackoverflow.com/questions/12301071/multidimensional-confidence-intervals

Usage:
>>> jointplot(M.rtr.trace(),M.mdot.trace(),xlabel='$\log \ r_{\\rm tr}$', ylabel='$\log \ \dot{m}$')
	"""
	import scipy

	# Generates 2D histogram for image
	histt, xt, yt = numpy.histogram2d(X, Y, bins=[binsim,binsim], normed=False)
	histt = numpy.transpose(histt)  # Beware: numpy switches axes, so switch back.

	# assigns correct proportions to subplots
	fig=pylab.figure()
	gs = pylab.GridSpec(2, 2, width_ratios=[3,1], height_ratios=[1,3], wspace=0.001, hspace=0.001)
	con=pylab.subplot(gs[2])
	histx=pylab.subplot(gs[0], sharex=con)
	histy=pylab.subplot(gs[3], sharey=con)
		
	# Image
	con.imshow(histt,extent=[xt[0],xt[-1], yt[0],yt[-1]],origin='lower',cmap=pylab.cm.gray_r,aspect='auto')

	# Overplot with error contours 1,2 sigma
	if contour==True:
		pdf = scipy.stats.gaussian_kde([X, Y])
		x,y = pylab.meshgrid(xt,yt)
		z = numpy.array(pdf.evaluate([x.flatten(),y.flatten()])).reshape(x.shape)
		# the [61,15] values were obtained by trial and error until the joint confidence 
		# contours matched the confidence intervals from the individual X,Y
		s=scipy.stats.scoreatpercentile(pdf(pdf.resample(1000)), [61,15])
		cs=con.contour(x,y,z, levels=s, extent=[x[0],x[-1], y[0],y[-1]], linestyles=['-','-','-'], colors=['black','blue'])
		# use dictionary in order to assign your own labels to the contours.
		#fmtdict = {s[0]:r'$1\sigma$',s[1]:r'$2\sigma$'}
		#con.clabel(cs, fmt=fmtdict, inline=True, fontsize=20)
		if xlabel!=None: con.set_xlabel(xlabel)
		if ylabel!=None: con.set_ylabel(ylabel)

	# X-axis histogram
	histx.hist(X, binsh, histtype='stepfilled',facecolor='lightblue')
	pylab.setp(histx.get_xticklabels(), visible=False)	# no X label
	pylab.setp(histx.get_yticklabels(), visible=False)	# no Y label
	# Vertical lines with median and 1sigma confidence
	yax=histx.set_ylim()
	histx.plot([numpy.median(X),numpy.median(X)],[yax[0],yax[1]],'k-',linewidth=2) # median
	xsd=scipy.stats.scoreatpercentile(X, [15.87,84.13])
	histx.plot([xsd[0],xsd[0]],[yax[0],yax[1]],'k--') # -1sd
	histx.plot([xsd[-1],xsd[-1]],[yax[0],yax[1]],'k--') # +1sd

	# Y-axis histogram
	histy.hist(Y, binsh, histtype='stepfilled', orientation='horizontal',facecolor='lightyellow')
	pylab.setp(histy.get_yticklabels(), visible=False)	# no Y label
	pylab.setp(histy.get_xticklabels(), visible=False)	# no X label
	# Vertical lines with median and 1sigma confidence
	xax=histy.set_xlim()
	histy.plot([xax[0],xax[1]],[numpy.median(Y),numpy.median(Y)],'k-',linewidth=2) # median
	ysd=scipy.stats.scoreatpercentile(Y, [15.87,84.13])
	histy.plot([xax[0],xax[1]],[ysd[0],ysd[0]],'k--') # -1sd
	histy.plot([xax[0],xax[1]],[ysd[-1],ysd[-1]],'k--') # +1sd







"""
LSD operations = lists, sets, dictionaries (and arrays)
=========================================================
"""

import numpy
import scipy


def cmset_and(x,y):
	"""
Usage:

>>> cmset_and(x,y)

returns the index of the elements of array x which are also present in the
array y. 

This is equivalent to using the IDL command

>>> botha=cmset_op(namea, 'AND', nameb, /index)

i.e. performs the same thing as the IDL routine `cmset_op <http://cow.physics.wisc.edu/~craigm/idl/idl.html>`_.
	"""
	
	idel=[]	# list of indexes of x elements which are also in y
	i=0
	for xx in x:
		if xx in y: idel.append(i)
		i=i+1
		
	return idel




def cmsetsort_and(x,y):
	"""
Usage:

>>> cmsetsort_and(x,y)

returning the index of the elements of array x which are also present in the
array y. 

The resulting elements have the same order as the ones in y. For 
instance, if you run

>>> i=cmsetsort_and(x,y)
>>> x[i]==y

will return an array of True, whereas if you used instead cmset_and it is
not guaranteed that all elements would match in x[i] and y.

Inherited from :func:`nemmen.cmset_and`.
	"""
	
	idel=[]	# list of indexes of x elements which are also in y
	i=0
	for yy in y:
		i=numpy.where(x==yy)
		idel.append( i[0].item() )
		
	return idel
	



def cmset_not(x,y):
	"""
Usage:

>>> cmset_not(x,y)

returning the index of the elements of array x which are not present in the
array y. 

This is equivalent to using the IDL command
SET = CMSET_OP(A, 'AND', /NOT2, B, /INDEX)   ; A but not B
i.e. performs the same thing as the IDL routine cmset_op from
http://cow.physics.wisc.edu/~craigm/idl/idl.html.
	"""
	
	idel=[]	# list of indexes of x elements which NOT in y
	i=0
	for xx in x:
		if xx not in y: idel.append(i)
		i=i+1
		
	return idel


	
def nanzero(x):
	"""
Set nan elements to zero in the array.
	"""
	# Index of nan elements
	i=numpy.where(numpy.isnan(x)==True)

	y=x.copy()
	y[i]=0.
	#y[i]=1e-20
	
	# Removes the nan elements
	return y




def delnan(x):
	"""
Remove nan elements from the array.
	"""
	# Index of nan elements
	i=numpy.where(numpy.isnan(x)==True)
	
	# Removes the nan elements
	return numpy.delete(x,i)

	


def delweird(x):
	"""
Remove nan or inf elements from the array.
	"""
	# Index of nan elements
	i=numpy.where( (numpy.isnan(x)==True) | (numpy.isinf(x)==True) )
	
	# Removes the nan elements
	return numpy.delete(x,i)


	
def replacevals(x,minval):
	"""
Replace all values in array x for which abs(x)<=minval with x=sign(x)*minval.
	"""
	i=numpy.where(numpy.abs(x)<=minval)
	y=x.copy()
	y[i]=numpy.sign(y[i])*minval

	return y



	
def search(xref, x):
	"""
Search for the element in an array x with the value nearest xref.
Piece of code based on http://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array

>>> i=search(xref, x)

:param xref: input number, array or list of reference values
:param x: input array
:returns: index of the x-elements with values nearest to xref:
	"""
	if numpy.size(xref)==1:
		i=(numpy.abs(x-xref)).argmin()
	else:
		i=[]

		for y in xref:
			i.append( (numpy.abs(x-y)).argmin() )
	        
	return i
	


def sortindex(x,**kwargs):
	"""
Returns the list of indexes, ordered according to the numerical value of each 
element of x.

:param x: input array or list.
:returns: list of element indexes.
	"""
	f=lambda i: x[i]

	return sorted( range(numpy.size(x)) , key=f,**kwargs)
	



def norm(x1,x2=None):
	"""
Normalizes x1. If also given as input x2, then normalizes x1 to x2.

:param x1: input array
:param x2: optional
:returns: normalized x1
	"""
	if x2 is None:
		return x1/x1.max()
	else:
		return x1*x2.max()/x1.max()





def uarray(x,errx):
	"""
With the new releases of the uncertainties and astropy.io.ascii (0.2.3, the
replacement of asciitable), if I try to create an uncertainties array with
the column of a table imported with ascii I run into trouble. For instance, 
if I use the sequence of commands below:

>>> import astropy.io.ascii as asciitable
>>> raff= asciitable.read('data/rafferty06.dat')
>>> m,errm=raff['mass'],raff['errm']
>>> mass=unumpy.uarray(m,errm)
>>> x=0.2*mass

I get the error message: 

>>> TypeError: unsupported operand type(s) for *: 'float' and 'Column'

which I can only assume is due to the new way ascii handles tables.

I created this method to use as a replacement for unumpy.uarray that handles
the tables created with astropy.io.ascii.

Usage is the same as uncertainties.unumpy.uarray.

:type x,errx: arrays created with astropy.io.ascii.
:returns: uncertainties array.
	"""
	import uncertainties.unumpy as unumpy

	x=numpy.array(x)
	errx=numpy.array(errx)

	return unumpy.uarray(x,errx)






	
def bootstrap(v):
	"""
Constructs Monte Carlo simulated data set using the
Bootstrap algorithm.                                                                                   

Usage:

>>> bootstrap(x)

where x is either an array or a list of arrays. If it is a
list, the code returns the corresponding list of bootstrapped 
arrays assuming that the same position in these arrays map the 
same "physical" object.

Rodrigo Nemmen, http://goo.gl/8S1Oo
	"""
	if type(v)==list:
		vboot=[]	# list of boostrapped arrays
		n=v[0].size
		iran=scipy.random.randint(0,n,n)	# Array of random indexes
		for x in v:	vboot.append(x[iran])
	else:	# if v is an array, not a list of arrays
		n=v.size
		iran=scipy.random.randint(0,n,n)	# Array of random indexes
		vboot=v[iran]
	
	return vboot
	



def regrid(x,y,z,xnew,ynew,method='cubic'):
	"""
Regrid 1D arrays (x,y,z) -- where z is some scalar field mapped at positions
x,y -- to a 2d array Z defined in the cartesian grids xnew,ynew (1D arrays with 
new grid).

For the interpolation method, choose nearest, linear or cubic.

>>> rho=regrid(d.x,d.y,d.rho,xnew,ynew)

.. todo:: need to create a 3d version of this method, paving the road for the 3d simulations.
	"""
	import scipy.interpolate

	# regrid the data to a nice cartesian grid
	Z = scipy.interpolate.griddata((x, y), z, (xnew[None,:], ynew[:,None]), method=method)

	# get rid of NaNs
	return nanzero(Z)




def crop(z, x,y, xmin, xmax, ymin, ymax, all=False):
	"""
Crops the image or 2D array, leaving only pixels inside the region
you define.

>>> Znew,Xnew,Ynew = crop(Z, X, Y, 0,10,-20,20)

where X,Y are 1D or 2D arrays, and Z is a 2D array.

:param z: 2d array 
:param x,y: 1d or 2d arrays. In the latter case, they should have the same shape as z
:param all: should I return cropped Z,X,Y or only Z?
:returns: Z_cropped, X_cropped, Y_cropped
	"""
	if x.ndim==1: # if x,y are 1D
		# Index tuples with elements that will be selected along each dimension
		i=numpy.where((x>=xmin) & (x<=xmax))	# x
		j=numpy.where((y>=ymin) & (y<=ymax))	# y

		# Defines new x and y arrays
		xnew,ynew=x[i],y[j]
		
		i,j=i[0],j[0]	# tuples -> arrays (for matrix slicing below)
		znew=z[j[0]:j[-1],i[0]:i[-1]]	# CAREFUL with the ordering of the indexes!
	elif x.ndim==2: # if x,y are 2D
		i=numpy.where((x[0,:]>=xmin) & (x[0,:]<=xmax))
		j=numpy.where((y[:,0]>=ymin) & (y[:,0]<=ymax))
		i,j=i[0],j[0]

		xnew=x[j[0]:j[-1],i[0]:i[-1]]
		ynew=y[j[0]:j[-1],i[0]:i[-1]]
		znew=z[j[0]:j[-1],i[0]:i[-1]]
	else: 
		print("Dimensions of the input arrays are inconsistent")
		return
	
	if all==False:
		return znew	
	else:
		return znew,xnew,ynew





def arrAvg(alist):
	"""
	Given a list of 1D or 2D arrays, this method computes their average, 
	returning an array with the same shape as the input.

	:param alist: list of arrays 
	:returns: average, std. dev. -- arrays with the same shape as the input arrays

	Usage:

	>>> avg=arrAvg([x,y,z])
	"""
	if alist[0].ndim==2:
		# join arrays together, creating 3D arrays where the third dimension is e.g. time
		# or whatever index for the different arrays you want to average
		arr=numpy.stack(alist,axis=2)

		# performs the average 
		return numpy.mean(arr,axis=2), numpy.std(arr,axis=2)
	elif alist[0].ndim==1:
		arr=numpy.stack(alist,axis=1)
		return numpy.mean(arr,axis=1), numpy.std(arr,axis=1)
	else:
		print("Dimensionality not supported")







"""
Dealing with (GR)(R)(M)HD simulations
========================================

- RAISHIN
- Pluto
- HARM

See jupyter notebooks "grmhd*" for examples on how to use this
module.

TODO:

- [ ] incorporate Pluto class from pluto-tools
- [ ]
"""

import numpy, scipy
import tqdm
import numpy as np




class Raishin:
	"""
Class that reads a RAISHIN VTK datafile and converts to numpy format.

Attributes of the object:

- x,y,z: 1D position arrays for the mesh
- rho: 1D density array
- vx,vy,vz: 1D velocity arrays
- p: 1D pressure
- bx,by,bz: 1D magnetic field arrays
- b2: 1D B^2

Define an empty object:

>>> o=nmmn.grmhd.Raishin()

Reads data from a VTK file, new attributes rho, p, vx, bx etc:

>>> o.vtk("ok200.vtk")

Saves data as an ASCII file with columns corresponding to variables:

>>> o.savetxt("ok200.dat")
	"""

	#def __init__(self):
	# does nothing for now



	def vtk(self, vtkfile):
		"""
	Given a VTK file created with the RAISHIN GRMHD code, this reads the
	data as numpy arrays.
		"""
		import re

		f = open(vtkfile,"r")
		#newf=open("tmp.dat","w")	# file that will hold coordinates

		# booleans that will tell the code when to stop reading
		# data for a given variable:
		# boold for density, boolp for pressure etc
		boolxyz,boold,boolp,boolvx,boolvy,boolvz,boolb2,boolbx,boolby,boolbz=False,False,False,False,False,False,False,False,False,False
		strx,stry,strz,strd,strp,strvx,strvy,strvz,strb2,strbx,strby,strbz='','','','','','','','','','','','' # string that holds values 

		for line in f:
			# gets dimensions
			if re.search(r'DIMENSIONS\s+\d+\s+\d+',line):
				s=re.findall(r'\s*\d+\s*',line.rstrip('\n'))
				self.nx=int(s[0])
				self.ny=int(s[1])
				self.nz=int(s[2])
				boolxyz=True
		    
			# gets arrays
			# these lines are important to tell python when to stop reading shit
			# it must be sequential
			if 'density' in line: 
				boolxyz=False
				boold=True
			if 'pressure' in line: 
				boold=False
				boolp=True
			if 'LorentzW1' in line:
				boolp=False
			if 'util^x' in line:
				boolvx=True
			if 'util^y' in line:
				boolvx=False
				boolvy=True    
			if 'util^z' in line:
				boolvy=False
				boolvz=True  
			if 'b^2' in line:
				boolvz=False
				boolb2=True    		          
			if 'bx' in line:
				boolb2=False
				boolbx=True
			if 'by' in line:
				boolbx=False
				boolby=True    
			if 'bz' in line: 
				boolby=False    
				boolbz=True
		    
			if boolxyz==True and re.search(r'-?\d+\.\d+E?[-+]?\d+\s+-?\d+\.\d+E?[-+]?\d+\s+-?\d+\.\d+E?[-+]?\d+',line):
				s=re.findall(r'\s*-?\d+\.\d+E?[-+]?\d+\s*',line.rstrip('\n'))
				strx=strx+s[0]
				stry=stry+s[1]
				strz=strz+s[2]
			if boold==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strd=strd+line
			if boolp==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strp=strp+line
			if boolvx==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strvx=strvx+line
			if boolvy==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strvy=strvy+line
			if boolvz==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strvz=strvz+line
			if boolb2==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strb2=strb2+line
			if boolbx==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strbx=strbx+line		        
			if boolby==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strby=strby+line
			if boolbz==True and re.search(r'-?\d+\.\d+E?[-+]?\d+',line):
				strbz=strbz+line

		# gets numpy arrays finally
		self.x=numpy.fromstring(strx, sep='\n')
		self.y=numpy.fromstring(stry, sep='\n')
		self.z=numpy.fromstring(strz, sep='\n')
		self.rho=numpy.fromstring(strd, sep='\n')
		self.p=numpy.fromstring(strp, sep='\n')
		self.vx=numpy.fromstring(strvx, sep='\n')
		self.vy=numpy.fromstring(strvy, sep='\n')
		self.vz=numpy.fromstring(strvz, sep='\n')
		self.b2=numpy.fromstring(strb2, sep='\n')
		self.bx=numpy.fromstring(strbx, sep='\n')
		self.by=numpy.fromstring(strby, sep='\n')
		self.bz=numpy.fromstring(strbz, sep='\n')

		# reads mesh positions 
		#self.x,self.y,self.z= numpy.loadtxt('tmp.dat',unpack=True,usecols=(0,1,2))

		# close files        
		f.close()
		#newf.close()




	def savetxt(self,outfile):
		"""Saves data as ASCII file """
		numpy.savetxt(outfile,numpy.transpose((self.x,self.y,self.z,self.rho,self.p,self.vx,self.vy,self.vz,self.bx,self.by,self.bz)))


	def savehdf5(self,outfile):
		"""
	Exports data as compressed HDF5. 7x less space than ASCII.
		"""
		import h5py

		with h5py.File(outfile, 'w') as hf:
			grid=hf.create_group('grid')
			grid.create_dataset('x', data=self.x, compression="gzip", compression_opts=9)
			grid.create_dataset('y', data=self.y, compression="gzip", compression_opts=9)
			grid.create_dataset('z', data=self.z, compression="gzip", compression_opts=9)    

			fields=hf.create_group('fields')
			fields.create_dataset('density', data=self.rho, compression="gzip", compression_opts=9)
			fields.create_dataset('pressure', data=self.p, compression="gzip", compression_opts=9)
			fields.create_dataset('vx', data=self.vx, compression="gzip", compression_opts=9)
			fields.create_dataset('vy', data=self.vy, compression="gzip", compression_opts=9)
			fields.create_dataset('vz', data=self.vz, compression="gzip", compression_opts=9)
			fields.create_dataset('bx', data=self.bx, compression="gzip", compression_opts=9)
			fields.create_dataset('by', data=self.by, compression="gzip", compression_opts=9)
			fields.create_dataset('bz', data=self.bz, compression="gzip", compression_opts=9)


	def savenumpy(self,outfile):
		"""
	Save data as binary Numpy file .npz. 3x less space than ASCII.
		"""
		numpy.savez(outfile,x=self.x,y=self.y,z=self.z,rho=self.rho,p=self.p,vx=self.vx,vy=self.vy,vz=self.vz,bx=self.bx,by=self.by,bz=self.bz)



	def regridAll(self,nboost=5):
		"""
	Regrid all RAISHIN data to a nice cartesian grid for plotting with
	python.

	:param nboost: factor of increase of number of grid points compared to 
		previous grid

	Usage:

	>>> d=nmmn.grmhd.Raishin()
	>>> d.vtk('ok100.vtk')
	>>> d.regridAll()

	Gets interpolated rho:

	>>> print(d.xc)

	TODO:
	- 3D version
	- parallel version
		"""
		#import lsd
		from . import lsd # py3

		# create two new arrays with spatial grid, with more points than the 
		# original grid
		nxnew=self.nx*nboost
		nynew=self.ny*nboost
		nznew=self.nz*nboost
		xnew=numpy.linspace(self.x.min(),round(self.x.max()),nxnew)
		ynew=numpy.linspace(self.y.min(),round(self.y.max()),nynew)
		znew=numpy.linspace(self.z.min(),round(self.z.max()),nznew)

		# 'c' is added to 2D array values
		self.xc,self.yc=numpy.meshgrid(xnew,ynew) # 2D
		self.xc1d,self.yc1d,self.zc1d=xnew,ynew,znew # 1D

		# bottleneck,
		self.rhoc=lsd.regrid(self.x,self.y,self.rho,xnew,ynew)
		self.pc=lsd.regrid(self.x,self.y,self.p,xnew,ynew)
		self.vxc=lsd.regrid(self.x,self.y,self.vx,xnew,ynew)
		self.vyc=lsd.regrid(self.x,self.y,self.vy,xnew,ynew)
		self.vzc=lsd.regrid(self.x,self.y,self.vz,xnew,ynew)
		self.bxc=lsd.regrid(self.x,self.y,self.bx,xnew,ynew)
		self.byc=lsd.regrid(self.x,self.y,self.by,xnew,ynew)
		self.bzc=lsd.regrid(self.x,self.y,self.bz,xnew,ynew)

		self.bc=numpy.sqrt(self.bxc**2+self.byc**2)
		self.vc=numpy.sqrt(self.vxc**2+self.vyc**2)
	


	def regrid(self,var,nboost=5):
		"""
	Regrid one specific RAISHIN array to a nice cartesian grid for 
	plotting with python. Note that RAISHIN's output is already in
	cartesian coordinates.

	:param var: array to be regridded e.g. d.rho
	:param nboost: factor of increase of number of grid points compared to 
		previous grid

	Usage:

	>>> d=nmmn.grmhd.Raishin()
	>>> d.vtk('ok100.vtk')
	>>> d.regrid(d.rho)

	TODO:
	- 3D version
	- parallel version
		"""
		#import lsd
		from . import lsd

		# create two new arrays with spatial grid, with more points than the 
		# original grid
		nxnew=self.nx*nboost
		nynew=self.ny*nboost
		xnew=numpy.linspace(self.x.min(),round(self.x.max()),nxnew)
		ynew=numpy.linspace(self.y.min(),round(self.y.max()),nynew)

		# 'c' is added to 2D array values
		self.xc,self.yc=numpy.meshgrid(xnew,ynew) # 2D
		self.xc1d,self.yc1d=xnew,ynew # 1D

		# bottleneck,
		return lsd.regrid(self.x,self.y,var,xnew,ynew)


	def regridsome(self,listarr,nboost=5):
		"""
	Regrid the selected arrays in the RAISHIN data to a nice cartesian 
	grid for plotting with python. Regridding only some of the arrays
	will, of course, speed up things.

	:param listarr: list of strings specifying the arrays to be regridded.
	  Options are: rho, p, v, b
	:param nboost: factor of increase of number of grid points compared to 
		previous grid

	Usage:

	>>> d=nmmn.grmhd.Raishin()
	>>> d.vtk('ok100.vtk')
	>>> d.regridsome(['rho','v'])

	TODO:
	- 3D version
	- parallel version
		"""
		#import lsd
		from . import lsd # py3

		# create two new arrays with spatial grid, with more points than the 
		# original grid
		nxnew=self.nx*nboost
		nynew=self.ny*nboost
		xnew=numpy.linspace(self.x.min(),round(self.x.max()),nxnew)
		ynew=numpy.linspace(self.y.min(),round(self.y.max()),nynew)

		# 'c' is added to 2D array values
		self.xc,self.yc=numpy.meshgrid(xnew,ynew) # 2D
		self.xc1d,self.yc1d=xnew,ynew # 1D

		# bottleneck
		if 'rho' in listarr:
			self.rhoc=lsd.regrid(self.x,self.y,self.rho,xnew,ynew)
		if 'p' in listarr: 
			self.pc=lsd.regrid(self.x,self.y,self.p,xnew,ynew)
		if 'v' in listarr:
			self.vxc=lsd.regrid(self.x,self.y,self.vx,xnew,ynew)
			self.vyc=lsd.regrid(self.x,self.y,self.vy,xnew,ynew)
			self.vzc=lsd.regrid(self.x,self.y,self.vz,xnew,ynew)
			self.vc=numpy.sqrt(self.vxc**2+self.vyc**2)
		if 'b' in listarr:
			self.bxc=lsd.regrid(self.x,self.y,self.bx,xnew,ynew)
			self.byc=lsd.regrid(self.x,self.y,self.by,xnew,ynew)
			self.bzc=lsd.regrid(self.x,self.y,self.bz,xnew,ynew)
			self.bc=numpy.sqrt(self.bxc**2+self.byc**2)




	def yt2d(self):
		"""
	Converts 2d arrays from raishin to the 3d format that is understood
	by the yt package. Make sure you used regridAll first.

	Inspired by this example: http://stackoverflow.com/questions/7372316/how-to-make-a-2d-numpy-array-a-3d-array
		"""
		self.x3d=self.xc.T[..., numpy.newaxis]
		self.y3d=self.yc.T[..., numpy.newaxis]

		self.rho3d=self.rhoc.T[..., numpy.newaxis]
		self.p3d=self.pc.T[..., numpy.newaxis]

		self.vx3d=self.vxc.T[..., numpy.newaxis]
		self.vy3d=self.vyc.T[..., numpy.newaxis]
		self.vz3d=self.vzc.T[..., numpy.newaxis]
		self.v3d=self.vc.T[..., numpy.newaxis]

		self.bx3d=self.bxc.T[..., numpy.newaxis]
		self.by3d=self.byc.T[..., numpy.newaxis]
		self.bz3d=self.bzc.T[..., numpy.newaxis]
		self.b3d=self.bc.T[..., numpy.newaxis]










class Harm:
	"""
Class that reads a HARM dump datafile and converts to numpy format
for plotting with matplotlib, mayavi etc. Heavily inspired/copied from
harm_script.py.

Reads data from a harm dump file:

>>> d=nmmn.grmhd.Harm("dump019")

The `d` object will then have several attributes related to the 
dump physical fields, grid and coordinate information.

Inspect densities:

>>> d.rho

Convert arrays to cartesian coordinates for plotting with matplotlib:

>>> rhonew,x,y=o.cartesian(o.rho)
>>> pcolormesh(x,y,log10(rhonew))
	"""

	def __init__(self, dump=None, gdump='gdump'):
		"""
		TODO:
		- [ ] input number of snapshot instead of filename
		"""
		self.dump=dump
		self.gdump=gdump

		if dump is not None:
			# read grid information
			self.read_file(gdump,type="gdump")

			# read dump file
			self.read_file(dump,type="dump")
		else:
			print("Please provide a dump file.")



	def read_file(self,dump,type=None,savedump=True,saverdump=False,noround=False):
		"""
	High-level function that reads either MPI or serial gdump's
		"""
		import os,sys
		import glob

		if type is None:
			if dump.startswith("dump"):
				type = "dump"
				print("Reading a dump file %s ..." % dump)
			elif dump.startswith("gdump2"):
				type = "gdump2"
				print("Reading a gdump2 file %s ..." % dump)
			elif dump.startswith("gdump"):
				type = "gdump"
				print("Reading a gdump file %s ..." % dump)
			elif dump.startswith("rdump"):
				type = "rdump"
				print("Reading a rdump file %s ..." % dump)
			elif dump.startswith("fdump"):
				type = "fdump"
				print("Reading a fdump file %s ..." % dump)
			else:
				print("Couldn't guess dump type; assuming it is a data dump")
				type = "dump"

	    #normal dump
		if os.path.isfile( "dumps/" + dump ):
			headerline = self.read_header("dumps/" + dump, returnheaderline = True)
			gd = self.read_body("dumps/" + dump,nx=self.N1+2*self.N1G,ny=self.N2+2*self.N2G,nz=self.N3+2*self.N3G,noround=1)
			if noround:
				res = self.data_assign(         gd,type=type,nx=self.N1+2*self.N1G,ny=self.N2+2*self.N2G,nz=self.N3+2*self.N3G)
			else:
				res = self.data_assign(myfloat(gd),type=type,nx=self.N1+2*self.N1G,ny=self.N2+2*self.N2G,nz=self.N3+2*self.N3G)
			return res

	    #MPI-type dump that is spread over many files
		else:
			flist = np.sort(glob.glob( "dumps/" + dump + "_[0-9][0-9][0-9][0-9]" ))
			if len(flist) == 0:
				print( "Could not find %s or its MPI counterpart" % dump )
				return
			sys.stdout.write( "Reading %s (%d files)" % (dump, len(flist)) )
			sys.stdout.flush()
			ndots = 10
			dndot = len(flist)/ndots
			if dndot == 0: dndot = 1
			for i,fname in enumerate(flist):
				#print( "Reading file %d out of %d..." % (i,len(flist)) )
				#header for each file might be different, so read each
				header = read_header(fname,issilent=1)
				if header is None:
					print( "Error reading header of %s, aborting..." % fname )
					return
				lgd = read_body(fname,nx=N1+2*N1G,ny=N2+2*N2G,nz=N3+2*N3G)
				#this gives an array of dimensions (-1,N1,N2,N3)+potentially ghost cells
				if 0 == i:
				#create full array: of dimensions, (-1,nx,ny,nz)
					fgd = np.zeros( (lgd.shape[0], nx+2*N1G, ny+2*N2G, nz+2*N3G), dtype=np.float32)
				if not type == "rdump":
					#construct full indices: ti, tj, tk
					#fti,ftj,ftk = mgrid[0:nx,0:ny,0:nz]
					lti,ltj,ltk = lgd[0:3,:,:].view();
					lti = np.int64(lti)
					ltj = np.int64(ltj)
					ltk = np.int64(ltk)
					fgd[:,lti+N1G,ltj+N2G,ltk+N3G] = lgd[:,:,:,:]
				else:
					print(starti,startj,startk)
					fgd[:,starti:starti+N1+2*N1G,startj:startj+N2+2*N2G,startk:startk+N3+2*N3G] = lgd[:,:,:,:]
				del lgd
				if i%dndot == 0:
					sys.stdout.write(".")
					sys.stdout.flush()
			res = data_assign(fgd,type=type,nx=nx+2*N1G,ny=ny+2*N2G,nz=nz+2*N3G)
			if savedump:
				#if the full dump file does not exist, create it
				dumpfullname = "dumps/" + dump
				if (type == "dump" or type == "gdump") and not os.path.isfile(dumpfullname):
					sys.stdout.write("Saving full dump to %s..." % dumpfullname)
					sys.stdout.flush()
					header[1] = header[4] #N1 = nx
					header[2] = header[5] #N2 = ny
					header[3] = header[6] #N3 = nz
					fout = open( dumpfullname, "wb" )
					#join header items with " " (space) as a glue
					#see http://stackoverflow.com/questions/12377473/python-write-versus-writelines-and-concatenated-strings
					#write it out with a new line char at the end
					fout.write(" ".join(header) + "\n")
					fout.flush()
					os.fsync(fout.fileno())
					#reshape the dump content
					gd1 = fgd.transpose(1,2,3,0)
					gd1.tofile(fout)
					fout.close()
					print( " done!" )
					if res is not None:
						return res
			return res

	def read_header(self,dump,issilent=True,returnheaderline=False):
		"""Read the header for the dump file"""
		# I am replacing all global variables below as attributes
		# of the object
		"""
		global t,nx,ny,nz,N1,N2,N3,N1G,N2G,N3G,starti,startj,startk,_dx1,_dx2,_dx3,a,gam,Rin,Rout,hslope,R0,ti,tj,tk,x1,x2,x3,r,h,ph,gcov,gcon,gdet,drdx,gn3,gv3,guu,gdd,dxdxp, games, startx1, startx2, startx3, tf, NPR, DOKTOT, BL
		global fractheta
		global fracphi
		global rbr
		global npow2
		global cpow2
		"""

		#read image
		fin = open( dump, "rb" )
		headerline = fin.readline()
		header = headerline.split()
		nheadertot = len(header)
		fin.close()
		# Creates object attributes
		if not dump.startswith("dumps/rdump"):
			if not issilent: print( "dump header: len(header) = %d" % len(header) )
			nheader = 45
			n = 0
			self.t = myfloat(np.float64(header[n])); n+=1
			#per tile resolution
			self.N1 = int(header[n]); n+=1
			self.N2 = int(header[n]); n+=1
			self.N3 = int(header[n]); n+=1
			#total resolution
			self.nx = int(header[n]); n+=1
			self.ny = int(header[n]); n+=1
			self.nz = int(header[n]); n+=1
			#numbers of ghost cells
			self.N1G = int(header[n]); n+=1
			self.N2G = int(header[n]); n+=1
			self.N3G = int(header[n]); n+=1
			self.startx1 = myfloat(float(header[n])); n+=1
			self.startx2 = myfloat(float(header[n])); n+=1
			self.startx3 = myfloat(float(header[n])); n+=1
			self._dx1=myfloat(float(header[n])); n+=1
			self._dx2=myfloat(float(header[n])); n+=1
			self._dx3=myfloat(float(header[n])); n+=1
			self.tf=myfloat(float(header[n])); n+=1
			self.nstep=myfloat(float(header[n])); n+=1
			self.a=myfloat(float(header[n])); n+=1
			self.gam=myfloat(float(header[n])); n+=1
			self.cour=myfloat(float(header[n])); n+=1
			self.DTd=myfloat(float(header[n])); n+=1
			self.DTl=myfloat(float(header[n])); n+=1
			self.DTi=myfloat(float(header[n])); n+=1
			self.DTr=myfloat(float(header[n])); n+=1
			self.DTr01=myfloat(float(header[n])); n+=1
			self.dump_cnt=myfloat(float(header[n])); n+=1
			self.image_cnt=myfloat(float(header[n])); n+=1
			self.rdump_cnt=myfloat(float(header[n])); n+=1
			self.rdump01_cnt=myfloat(float(header[n])); n+=1
			self.dt=myfloat(float(header[n])); n+=1
			self.lim=myfloat(float(header[n])); n+=1
			self.failed=myfloat(float(header[n])); n+=1
			self.Rin=myfloat(float(header[n])); n+=1
			self.Rout=myfloat(float(header[n])); n+=1
			self.hslope=myfloat(float(header[n])); n+=1
			self.R0=myfloat(float(header[n])); n+=1
			self.NPR=int(header[n]); n+=1
			self.DOKTOT=int(header[n]); n+=1
			self.fractheta = myfloat(header[n]); n+=1
			self.fracphi   = myfloat(header[n]); n+=1
			self.rbr       = myfloat(header[n]); n+=1
			self.npow2     = myfloat(header[n]); n+=1
			self.cpow2     = myfloat(header[n]); n+=1
			self.BL = myfloat(header[n]); n+=1
		else:
			print("rdump header")
			nheader = 46
			n = 0
			#per tile resolution
			self.N1 = int(header[n]); n+=1
			self.N2 = int(header[n]); n+=1
			self.N3 = int(header[n]); n+=1
			#total resolution
			self.nx = int(header[n]); n+=1
			self.ny = int(header[n]); n+=1
			self.nz = int(header[n]); n+=1
			#numbers of ghost cells
			self.N1G = int(header[n]); n+=1
			self.N2G = int(header[n]); n+=1
			self.N3G = int(header[n]); n+=1
			#starting indices
			self.starti = int(header[n]); n+=1
			self.startj = int(header[n]); n+=1
			self.startk = int(header[n]); n+=1
			self.t = myfloat(header[n]); n+=1
			self.tf = myfloat(header[n]); n+=1
			self.nstep = int(header[n]); n+=1
			self.a = myfloat(header[n]); n+=1
			self.gam = myfloat(header[n]); n+=1
			self.game = myfloat(header[n]); n+=1
			self.game4 = myfloat(header[n]); n+=1
			self.game5 = myfloat(header[n]); n+=1
			self.cour = myfloat(header[n]); n+=1
			self.DTd = myfloat(header[n]); n+=1
			self.DTl = myfloat(header[n]); n+=1
			self.DTi = myfloat(header[n]); n+=1
			self.DTr = myfloat(header[n]); n+=1
			self.DTr01 = myfloat(header[n]); n+=1
			self.dump_cnt = myfloat(header[n]); n+=1
			self.image_cnt = myfloat(header[n]); n+=1
			self.rdump_cnt = myfloat(header[n]); n+=1
			self.rdump01_cnt=myfloat(float(header[n])); n+=1
			self.dt = myfloat(header[n]); n+=1
			self.lim = myfloat(header[n]); n+=1
			self.failed = myfloat(header[n]); n+=1
			self.Rin = myfloat(header[n]); n+=1
			self.Rout = myfloat(header[n]); n+=1
			self.hslope = myfloat(header[n]); n+=1
			self.R0 = myfloat(header[n]); n+=1
			self.fractheta = myfloat(header[n]); n+=1
			self.fracphi = myfloat(header[n]); n+=1
			self.rbr = myfloat(header[n]); n+=1
			self.npow2 = myfloat(header[n]); n+=1
			self.cpow2 = myfloat(header[n]); n+=1
			self.tdump = myfloat(header[n]); n+=1
			self.trdump = myfloat(header[n]); n+=1
			self.timage = myfloat(header[n]); n+=1
			self.tlog  = myfloat(header[n]); n+=1

		if n != nheader or n != nheadertot:
			print("Wrong number of elements in header: nread = %d, nexpected = %d, nototal = %d: incorrect format?"% (n, nheader, nheadertot) )
			return headerline
		if returnheaderline:
			return headerline
		else:
			return header
	            
	def read_body(self,dump,nx=None,ny=None,nz=None,noround=False):
		fin = open( dump, "rb" )
		header = fin.readline()
		if dump.startswith("dumps/rdump"):
			dtype = np.float64
			body = np.fromfile(fin,dtype=dtype,count=-1)
			gd = body.view().reshape((self.nx,self.ny,self.nz,-1), order='C')
			if noround:
				gd=gd.transpose(3,0,1,2)
			else:
				gd=myfloat(gd.transpose(3,0,1,2))
		elif dump.startswith("dumps/gdump2"):
			dtype = np.float64
			body = np.fromfile(fin,dtype=dtype,count=-1)
			gd = body.view().reshape((self.nx,self.ny,self.nz,-1), order='C')
			if noround:
				gd=gd.transpose(3,0,1,2)
			else:
				gd=myfloat(gd.transpose(3,0,1,2))
		elif dump.startswith("dumps/fdump"):
			dtype = np.int64
			body = np.fromfile(fin,dtype=dtype,count=-1)
			gd = body.view().reshape((-1,self.nz,self.ny,self.nx), order='F')
			gd=myfloat(gd.transpose(0,3,2,1))
		else:
			dtype = np.float32
			body = np.fromfile(fin,dtype=dtype,count=-1)
			gd = body.view().reshape((-1,self.nz,self.ny,self.nx), order='F')
			gd=myfloat(gd.transpose(0,3,2,1))
		return gd

	def data_assign(self,gd,type=None,**kwargs):
		if type is None:
			print("Please specify data type")
			return
		if type == "gdump":
			self.gdump_assign(gd,**kwargs)
			return None
		elif type == "gdump2":
			self.gdump2_assign(gd,**kwargs)
			return None
		elif type == "dump":
			self.dump_assign(gd,**kwargs)
			return None
		elif type == "rdump":
			gd = self.rdump_assign(gd,**kwargs)
			return gd
		elif type == "fdump":
			gd = self.fdump_assign(gd,**kwargs)
			return gd
		else:
			print("Unknown data type: %s" % type)
			return gd
	    
	def gdump_assign(self,gd,**kwargs):
		#global t,nx,ny,nz,N1,N2,N3,_dx1,_dx2,_dx3,a,gam,Rin,Rout,hslope,R0,ti,tj,tk,x1,x2,x3,r,h,ph,gcov,gcon,gdet,drdx,gn3,gv3,guu,gdd,dxdxp, games
		self.nx = kwargs.pop("nx",self.nx)
		self.ny = kwargs.pop("ny",self.ny)
		self.nz = kwargs.pop("nz",self.nz)
		self.ti,self.tj,self.tk,self.x1,self.x2,self.x3,self.r,self.h,self.ph = gd[0:9,:,:].view();  n = 9
		self.gv3 = gd[n:n+16].view().reshape((4,4,self.nx,self.ny,self.nz),order='F').transpose(1,0,2,3,4); n+=16
		self.gn3 = gd[n:n+16].view().reshape((4,4,self.nx,self.ny,self.nz),order='F').transpose(1,0,2,3,4); n+=16
		self.gcov = self.gv3
		self.gcon = self.gn3
		self.guu = self.gn3
		self.gdd = self.gv3
		self.gdet = gd[n]; n+=1
		self.drdx = gd[n:n+16].view().reshape((4,4,self.nx,self.ny,self.nz),order='F').transpose(1,0,2,3,4); n+=16
		self.dxdxp = self.drdx
		if n != gd.shape[0]:
			print("rd: WARNING: nread = %d < ntot = %d: incorrect format?" % (n, gd.shape[0]) )
			return 1
		return 0

	def gdump2_assign(self,gd,**kwargs):
		#global t,nx,ny,nz,N1,N2,N3,_dx1,_dx2,_dx3,a,gam,Rin,Rout,hslope,R0,ti,tj,tk,x1,x2,x3,gdet,games,rf1,hf1,phf1,rf2,hf2,phf2,rf3,hf3,phf3,rcorn,hcord,phcorn,re1,he1,phe1,re2,he2,phe2,re3,he3,phe3
		self.nx = kwargs.pop("nx",self.nx)
		self.ny = kwargs.pop("ny",self.ny)
		self.nz = kwargs.pop("nz",self.nz)
		self.ti,self.tj,self.tk,self.x1,self.x2,self.x3 = gd[0:6,:,:].view();  n = 6
		self.rf1,self.hf1,self.phf1,self.rf2,self.hf2,self.phf2,self.rf3,self.hf3,self.phf3 = gd[0:9,:,:].view();  n += 9
		self.rcorn,self.hcord,self.phcorn,self.rcent,self.hcent,self.phcen = gd[0:6,:,:].view();  n += 6
		self.re1,self.he1,self.phe1,self.re2,self.he2,self.phe2,self.re3,self.he3,self.phe3 = gd[0:9,:,:].view();  n += 9
		self.gdet = gd[n]; n+=1
		if n != gd.shape[0]:
			print("rd: WARNING: nread = %d < ntot = %d: incorrect format?" % (n, gd.shape[0]) )
			return 1
		return 0

	#read in a dump file
	def dump_assign(self,gd,**kwargs):
		#global t,nx,ny,nz,_dx1,_dx2,_dx3,gam,hslope,a,R0,Rin,Rout,ti,tj,tk,x1,x2,x3,r,h,ph,rho,ug,vu,B,pg,cs2,Sden,U,gdetB,divb,uu,ud,bu,bd,v1m,v1p,v2m,v2p,gdet,bsq,gdet,alpha,rhor, ktot, pg
		self.nx = kwargs.pop("nx",self.nx)
		self.ny = kwargs.pop("ny",self.ny)
		self.nz = kwargs.pop("nz",self.nz)
		self.ti,self.tj,self.tk,self.x1,self.x2,self.x3,self.r,self.h,self.ph,self.rho,self.ug = gd[0:11,:,:].view(); n = 11
		self.pg = (self.gam-1)*self.ug
		#lrho=np.log10(self.rho)
		self.vu=np.zeros_like(gd[0:4])
		self.B=np.zeros_like(gd[0:4])
		self.vu[1:4] = gd[n:n+3]; n+=3
		self.B[1:4] = gd[n:n+3]; n+=3
		#if total entropy equation is evolved (on by default)
		if self.DOKTOT == 1:
			self.ktot = gd[n]; n+=1
		self.divb = gd[n]; n+=1
		self.uu = gd[n:n+4]; n+=4
		self.ud = gd[n:n+4]; n+=4
		self.bu = gd[n:n+4]; n+=4
		self.bd = gd[n:n+4]; n+=4
		self.bsq = mdot(self.bu,self.bd)
		self.v1m,self.v1p,self.v2m,self.v2p,self.v3m,self.v3p=gd[n:n+6]; n+=6
		self.gdet=gd[n]; n+=1
		self.rhor = 1+(1-self.a**2)**0.5
		if hasattr(self, 'guu'):
		#if "guu" in globals():
			#lapse
			alpha = (-self.guu[0,0])**(-0.5)
		if n != gd.shape[0]:
			print("rd: WARNING: nread = %d < ntot = %d: incorrect format?" % (n, gd.shape[0]) )
			return 1
		return 0

	def rdump_assign(self,gd,**kwargs):
		#global t,nx,ny,nz,_dx1,_dx2,_dx3,gam,hslope,a,R0,Rin,Rout,ti,tj,tk,x1,x2,x3,r,h,ph,rho,ug,vu,B,pg,cs2,Sden,U,gdetB,divb,uu,ud,bu,bd,v1m,v1p,v2m,v2p,gdet,bsq,gdet,alpha,rhor, ktot, Ttot, game, qisosq, pflag, qisodotb, kel, uelvar, Tel4, Tel5,Teldis, Tels, kel4, kel5,ugel,ugeldis, ugcon, sel, ugscon, ugel4, ugel5,stot, uelvar, Telvar, Tsel, sel, ugels, games, phi, keldis, phihat,csphib,lrho
		self.nx = kwargs.pop("nx",self.nx)
		self.ny = kwargs.pop("ny",self.ny)
		self.nz = kwargs.pop("nz",self.nz)
		n = 0
		self.rho = gd[n]; n+=1
		self.ug = gd[n]; n+=1
		self.vu=np.zeros_like(gd[0:4])
		self.B=np.zeros_like(gd[0:4])
		self.vu[1:4] = gd[n:n+3]; n+=3
		self.B[1:4] = gd[n:n+3]; n+=3
		# if n != gd.shape[0]:
		#     print("rd: WARNING: nread = %d < ntot = %d: incorrect format?" % (n, gd.shape[0]) )
		#     return 1
		return gd

	def fdump_assign(self,gd,**kwargs):
		#global t,nx,ny,nz,_dx1,_dx2,_dx3,gam,hslope,a,R0,Rin,Rout,ti,tj,tk,x1,x2,x3,r,h,ph,rho,ug,vu,B,pg,cs2,Sden,U,gdetB,divb,uu,ud,bu,bd,v1m,v1p,v2m,v2p,gdet,bsq,gdet,alpha,rhor, ktot, Ttot, game, qisosq, pflag, qisodotb, kel, uelvar, Tel4, Tel5,Teldis, Tels, kel4, kel5,ugel,ugeldis, ugcon, sel, ugscon, ugel4, ugel5,stot, uelvar, Telvar, Tsel, sel, ugels, games, phi, keldis, phihat,csphib,lrho,fail
		self.nx = kwargs.pop("nx",self.nx)
		self.ny = kwargs.pop("ny",self.ny)
		self.nz = kwargs.pop("nz",self.nz)
		self.fail = gd
		return gd



	def cartesian_sasha(self,yourvar,k=0,xy=1,xcoord=None,ycoord=None,symmx=0,mirrorx=0,mirrory=0):
		"""
	This was adapted from `plc` at `harm_script.py`, atchekho/harmpi.
	Need to understand this better. 

	- k: 3D slice (if applicable)
		"""
		r=self.r 
		h=self.h
		ph=self.ph

		# avoids trouble
		myvar=yourvar.copy()

		if np.abs(xy)==1:
			if xcoord is None: xcoord = r * np.sin(h)
			if ycoord is None: ycoord = r * np.cos(h)
			if mirrory: ycoord *= -1
			if mirrorx: xcoord *= -1
		if xcoord is not None and ycoord is not None:
			xcoord = xcoord[:,:,None] if xcoord.ndim == 2 else xcoord[:,:,k:k+1]
			ycoord = ycoord[:,:,None] if ycoord.ndim == 2 else ycoord[:,:,k:k+1]
		if np.abs(xy)==1 and symmx:
			if myvar.ndim == 2:
				myvar = myvar[:,:,None] if myvar.ndim == 2 else myvar[:,:,k:k+1]
				myvar=np.concatenate((myvar[:,::-1],myvar),axis=1)
				xcoord=np.concatenate((-xcoord[:,::-1],xcoord),axis=1)
				ycoord=np.concatenate((ycoord[:,::-1],ycoord),axis=1)
			else:
				if myvar.shape[-1] > 1: 
					symmk = (k+self.nz/2)%self.nz 
				else: 
					symmk = k
				myvar=np.concatenate((myvar[:,self.ny-1:self.ny,k:k+1],myvar[:,::-1,symmk:symmk+1],myvar[:,:,k:k+1]),axis=1)
				xcoord=np.concatenate((xcoord[:,self.ny-1:self.ny,k:k+1],-xcoord[:,::-1],xcoord),axis=1)
				ycoord=np.concatenate((ycoord[:,self.ny-1:self.ny,k:k+1],ycoord[:,::-1],ycoord),axis=1)
		elif np.abs(xy) == 2 and symmx:
			#if fracphi == 0.5 done in a robust way
			if get_fracphi() < 0.75:
				r1 = np.concatenate((r,r,r[...,0:1]),axis=2)
				ph1 = np.concatenate((ph,ph+np.pi,ph[...,0:1]+2*np.pi),axis=2)
				myvar = np.concatenate((myvar,myvar,myvar[...,0:1]),axis=2)
			else:
				r1 = np.concatenate((r,r[...,0:1]),axis=2)
				ph1 = np.concatenate((ph,ph[...,0:1]+2*np.pi),axis=2)
				myvar = np.concatenate((myvar,myvar[...,0:1]),axis=2)
			xcoord=(r1*cos(ph1))[:,self.ny/2,:,None]
			ycoord=(r1*sin(ph1))[:,self.ny/2,:,None]
			myvar = myvar[:,self.ny/2,:,None]
		else:
			if myvar.ndim == 2:
				myvar = myvar[:,:,None]  
			else:
				myvar[:,:,k:k+1]

		return myvar[:,:,k],xcoord[:,:,k],ycoord[:,:,k]


	def cartesian(self,myvar=None):
		"""
	Computes cartesian coordinates.

	Arguments:

	- k: 3D slice (if applicable)
		"""
		r=self.r 
		th=self.h
		phi=self.ph

		if self.rho.shape[2]>1: # 3D
			self.x=r*numpy.sin(th)*numpy.cos(phi)
			self.y=r*numpy.sin(th)*numpy.sin(phi)
			self.z=r*numpy.cos(th)
		else: # 2D
			x=r*numpy.sin(th)
			y=r*numpy.cos(th)

			self.x=x[:,:,0]
			self.y=y[:,:,0]

			if myvar is not None:
				return myvar[:,:,0]













def myfloat(f,acc=1): # Sasha
	""" acc=1 means np.float32, acc=2 means np.float64 """
	if acc==1:
		return( np.float32(f) )
	else:
		return( np.float64(f) )

def mdot(a,b):
	"""
    Computes a contraction of two tensors/vectors.  Assumes
    the following structure: tensor[m,n,i,j,k] OR vector[m,i,j,k], 
    where i,j,k are spatial indices and m,n are variable indices. 
	"""
	if (a.ndim == 3 and b.ndim == 3) or (a.ndim == 4 and b.ndim == 4):
		c = (a*b).sum(0)
	elif a.ndim == 5 and b.ndim == 4:
		c = np.empty(np.maximum(a[:,0,:,:,:].shape,b.shape),dtype=b.dtype)
		for i in range(a.shape[0]):
			c[i,:,:,:] = (a[i,:,:,:,:]*b).sum(0)
	elif a.ndim == 4 and b.ndim == 5:
		c = np.empty(np.maximum(b[0,:,:,:,:].shape,a.shape),dtype=a.dtype)
		for i in range(b.shape[1]):
			c[i,:,:,:] = (a*b[:,i,:,:,:]).sum(0)
	elif a.ndim == 5 and b.ndim == 5:
		c = np.empty((a.shape[0],b.shape[1],a.shape[2],a.shape[3],max(a.shape[4],b.shape[4])),dtype=a.dtype)
		for i in range(c.shape[0]):
			for j in range(c.shape[1]):
				c[i,j,:,:,:] = (a[i,:,:,:,:]*b[:,j,:,:,:]).sum(0)
	elif a.ndim == 5 and b.ndim == 6:
		c = np.empty((a.shape[0],b.shape[1],b.shape[2],max(a.shape[2],b.shape[3]),max(a.shape[3],b.shape[4]),max(a.shape[4],b.shape[5])),dtype=a.dtype)
		for mu in range(c.shape[0]):
			for k in range(c.shape[1]):
				for l in range(c.shape[2]):
					c[mu,k,l,:,:,:] = (a[mu,:,:,:,:]*b[:,k,l,:,:,:]).sum(0)
	else:
		raise Exception('mdot', 'wrong dimensions')
	return c


				
def fixminus(x):
	"""
	Replace nonphysical, negative values in array *x* with the corresponding
	positive numerical values. Returns modified array. Does not 
	touch original array.
	"""
	i=numpy.where(x<0)
	z=x.copy()
	z[i]=numpy.abs(x[i])

	return z




def wolframplot(infile,outfile,script="/Users/nemmen/work/software/mathematica/raishin.wl"):
	"""
Makes a pretty plot of density field and B vector field of RAISHIN
data using the Wolfram Language.

Make sure you point to the appropriate Wolfram script.

- infile: input RAISHIN ASCII file generated with Raishin.vtk above, e.g. ok200.dat
- outfile: format that will produced, e.g. ok200.png
	"""
	import subprocess
	cmd="wolframscript -script "+script+" "+infile+" "+outfile
	subprocess.call(cmd.split())





"""
Methods to handle Fermi LAT data
=====================================

Handle output of Fermi analysis.
"""

import os, sys
import pylab, numpy






def getconfig(configfile="analysis.conf"):
    """
Reads config file and gets parameters.
    """
    import configobj
    config = configobj.ConfigObj(configfile)

    # gets useful things 
    folder = config['out']
    cmapfile=folder+"/"+config['target']['name']+'_'+config['file']['tag']+'_CountMap.fits'
    modelmap=folder+"/"+config['target']['name']+'_'+config['file']['tag']+'_ModelMap.fits'
    residuals=folder+"/"+config['target']['name']+'_Residual_Model_cmap.fits'
    images = [residuals, modelmap, cmapfile]
    tsdir=folder+"/TSMap/"
    tsfits=folder+"/"+config['target']['name']+'_'+config['file']['tag']+'_TSMap.npz'  # tsmap filename

    # create dictionary with useful values
    useful = {'folder': folder, 'cmapfile': cmapfile, 'modelmap':modelmap, 'residuals':residuals, 'images':images, 'tsdir':tsdir, 'tsfits':tsfits}

    return config, useful





def plotMaps(configfile="analysis.conf",cmap=None):
    """
Given the directory where the results of an Enrico analysis are 
located (enrico_sed), this method plots the different count maps
(observed, model and residuals).

This is the fixed version of the script plotMaps.py from the Enrico
distribution. 

:param configdir: directory with "/" at the end that points to the place
where Enrico output files are located.
:returns: image file SOURCE_Maps.png in the source folder.
    """
    import astropy, astropy.io.fits, astropy.wcs
    import aplpy

    config,c=getconfig(configfile)

    def set_hgps_style(f):
        """Set HGPS style for a f = aplpy.FITSFigure"""
        f.ticks.set_xspacing(2)
        f.ticks.set_yspacing(2)
        f.ticks.set_linewidth(1.5)
        f.tick_labels.set_xformat('dd')
        f.tick_labels.set_yformat('dd')
        f.tick_labels.set_style('colons')
        f.axis_labels.set_xtext('Right Ascension (deg)')
        f.axis_labels.set_ytext('Declination (deg)')

    # Determine image center and width / height
    dpi = 2000
    header = astropy.io.fits.getheader(c['cmapfile'])
    wcs = astropy.wcs.WCS(header)
    header['NAXIS1'] / dpi
    header['NAXIS2'] / dpi
    lon, lat = header['NAXIS1'] / 2., header['NAXIS2'] / 2.
    x_center, y_center = wcs.wcs_pix2world(lon, lat, 0)
    radius = header['CDELT2'] * header['NAXIS2'] / 2.

    # Computing the sub-figure sizes is surprisingly hard
    figsize=(5, 15)
    figure = pylab.figure(figsize=figsize)
    axis_ratio = figsize[0] / float(figsize[1])
    edge_margin_x = 0.12
    edge_margin_y = edge_margin_x * axis_ratio
    edge_margin_x_up = 0.01
    edge_margin_y_up = edge_margin_x_up * axis_ratio
    inner_margin_x = 0.1
    inner_margin_y = inner_margin_x * axis_ratio
    size_x = (1 - edge_margin_x - edge_margin_x_up)
    size_y = (1 - edge_margin_y - edge_margin_y_up - 2 * inner_margin_y) / 3

    for i, image in enumerate(c['images']):
        subplot = [edge_margin_x, edge_margin_y + i * (size_y + inner_margin_y), size_x, size_y]
        f = aplpy.FITSFigure(image, figure=figure, subplot=subplot)
        f.recenter(x_center, y_center, 0.95 * radius)
        set_hgps_style(f)
        f.show_colorscale(stretch='power', exponent=1, cmap=cmap)
        #f.show_colorbar()
        if i==0: f.show_markers(x_center, y_center, edgecolor='Black', s=400.)
        if i==1: 
            f.show_markers(x_center, y_center, edgecolor='White', s=400.)
            f.show_regions(c['folder']+'/Roi_model.reg')
        if i==2: f.show_markers(x_center, y_center, edgecolor='Black', s=400.)

    plotfile =c['folder']+"/"+config['target']['name']+"_Maps.png"
    print('Writing {}'.format(plotfile))
    figure.savefig(plotfile)


def plotMap(counts,out='tmp.png',cmap=None,roi=None):
    """
This method plots some count map. Inherited from the method plotMaps
above.  

:param counts: FITS file with counts to be plotted
:param cmap: desired colormap for image
:param roi: DS9 region file if present, otherwise leave None
:param out: output filename and format
:returns: image file 'tmp.png' in the source folder.
    """
    import astropy, astropy.io.fits, astropy.wcs
    import aplpy

    def set_hgps_style(f):
        """Set HGPS style for a f = aplpy.FITSFigure"""
        f.ticks.set_xspacing(2)
        f.ticks.set_yspacing(2)
        f.ticks.set_linewidth(1.5)
        f.tick_labels.set_xformat('dd')
        f.tick_labels.set_yformat('dd')
        f.tick_labels.set_style('colons')
        f.axis_labels.set_xtext('Right Ascension (deg)')
        f.axis_labels.set_ytext('Declination (deg)')

    # Determine image center and width / height
    dpi = 2000
    header = astropy.io.fits.getheader(counts)
    wcs = astropy.wcs.WCS(header)
    header['NAXIS1'] / dpi
    header['NAXIS2'] / dpi
    lon, lat = header['NAXIS1'] / 2., header['NAXIS2'] / 2.
    x_center, y_center = wcs.wcs_pix2world(lon, lat, 0)
    radius = header['CDELT2'] * header['NAXIS2'] / 2.

    f = aplpy.FITSFigure(counts)
    f.recenter(x_center, y_center, 0.95 * radius)
    set_hgps_style(f)
    f.show_colorscale(stretch='power', exponent=1, cmap=cmap)
    f.show_colorbar()
    f.show_markers(x_center, y_center, edgecolor='White', s=400.)
    if roi!=None: f.show_regions(roi)
   
    print('Writing {}'.format(out))
    pylab.savefig(out)
    
    




def plotTSmap(configfile="analysis.conf") :
    """ From Enrico/tsmap.py.

    Gather the results of the evaluation of 
    each pixel and fill a fits file"""

    # gets parameters from the Enrico configuration file
    config,c=getconfig(configfile)

    # Read the cmap produced before to get the grid for the TS map
    npix = int(config['TSMap']['npix'])
    ts=numpy.zeros((npix,npix))
    ra=ts.copy()
    dec=ts.copy()

    import string # read the results
    for i in range(npix):
        for j in range(npix):
            try : 
                lines = open(c['tsdir']+PixelFile(i,j),"r").readlines()
                tsval = float(string.split(lines[0])[2])    # TS value
                raval = float(string.split(lines[0])[0])    # RA
                decval = float(string.split(lines[0])[1])   # dec
            except :
                print("Cannot find, open or read "+c['tsdir']+PixelFile(i,j))
                Value = 0.
            ts[i,j] = tsval
            ra[i,j] = raval
            dec[i,j] = decval


    # save file
    numpy.savez(c['tsfits'], TS=ts,ra=ra,dec=dec)
    print("TS Map saved in "+c['tsfits'])   

    return ra,dec,ts 
    
    


def PixelFile(i,j):
    """ return the name of a file where the result of 1 pixel 
    evaluation will be stored

    From enrico/tsmap.py.
    """
    return 'Pixel_'+str(i)+'_'+str(j)





def converttime(y):
    """
Converts from MET Fermi time to a python datetime tuple.

>>> convertyear(386817828)

returns datetime.datetime(2013, 4, 5, 1, 23, 48).

:param y: a float or array of floats
:returns: a datetime structure (or list) with the date corresponding to the input float or array

y is the amount of seconds since 2001.0 UTC. Beware that my method
may assume that all years have 365 days.

References: 

- http://stackoverflow.com/questions/19305991/convert-fractional-years-to-a-real-date-in-python
- http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/xTime/xTime.pl
    """
    import datetime

    t0=2001

    if numpy.size(y)==1:    # if input is float
        day_one = datetime.datetime(t0,1,1) # initial time MET
        d = datetime.timedelta(seconds=y) # dt since t0
        date = d + day_one
    else:   # if input is list/array
        date=[]

        for i, yi in enumerate(y): 
            day_one = datetime.datetime(t0,1,1)
            d = datetime.timedelta(seconds=yi)
            date.append(d + day_one)

    return date





def starttime(t):
    """
Fixes the start time of observations taken with LAT. This assumes that the
time array in your light curve is in days already, and the beginning of
your analysis coincides with t0 for LAT.

:param t: array with input times given in days, with t0 coinciding with the beginning of LAT
:returns: t (days), t (years)
    """
    import pendulum

    # start of observations by Fermi
    tp=pendulum.create(2008,8,4,15,43,37)-pendulum.create(2008,1,1)
    t0=tp.days # how many days since 2008 for the beginning of Fermi observations

    t=t-t[0]+t0 # fixes t array 
    ty=2008.+t/365. # time in years

    return t,ty


